@@ -1,5 +1,6 @@
+
 # -*- test-case-name: twisted.internet.test.test_endpoints -*-
-# Copyright (c) 2007-2010 Twisted Matrix Laboratories.
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 """
 Implementations of L{IStreamServerEndpoint} and L{IStreamClientEndpoint} that
@@ -11,34 +12,35 @@ parsed by the L{clientFromString} and L{serverFromString} functions.
 @since: 10.1
 """
 
+import os, socket
+
 from zope.interface import implements, directlyProvides
 import warnings
 
-from twisted.internet import interfaces, defer, error
+from twisted.internet import interfaces, defer, error, fdesc, threads
 from twisted.internet.protocol import ClientFactory, Protocol
-from twisted.plugin import getPlugins
+from twisted.plugin import IPlugin, getPlugins
 from twisted.internet.interfaces import IStreamServerEndpointStringParser
 from twisted.internet.interfaces import IStreamClientEndpointStringParser
 from twisted.python.filepath import FilePath
-
+from twisted.python.systemd import ListenFDs
+from twisted.internet.abstract import isIPv6Address
+from twisted.internet import stdio
+from twisted.internet.stdio import PipeAddress
 
 
 __all__ = ["clientFromString", "serverFromString",
-           "TCP4ServerEndpoint", "TCP4ClientEndpoint",
+           "TCP4ServerEndpoint", "TCP6ServerEndpoint",
+           "TCP4ClientEndpoint", "TCP6ClientEndpoint",
            "UNIXServerEndpoint", "UNIXClientEndpoint",
-           "SSL4ServerEndpoint", "SSL4ClientEndpoint"]
+           "SSL4ServerEndpoint", "SSL4ClientEndpoint",
+           "AdoptedStreamServerEndpoint", "StandardIOEndpoint"]
 
 
 class _WrappingProtocol(Protocol):
     """
     Wrap another protocol in order to notify my user when a connection has
     been made.
-
-    @ivar _connectedDeferred: The L{Deferred} that will callback
-        with the C{wrappedProtocol} when it is connected.
-
-    @ivar _wrappedProtocol: An L{IProtocol} provider that will be
-        connected.
     """
 
     def __init__(self, connectedDeferred, wrappedProtocol):
@@ -52,9 +54,20 @@ class _WrappingProtocol(Protocol):
         self._connectedDeferred = connectedDeferred
         self._wrappedProtocol = wrappedProtocol
 
-        if interfaces.IHalfCloseableProtocol.providedBy(
-            self._wrappedProtocol):
-            directlyProvides(self, interfaces.IHalfCloseableProtocol)
+        for iface in [interfaces.IHalfCloseableProtocol,
+                      interfaces.IFileDescriptorReceiver]:
+            if iface.providedBy(self._wrappedProtocol):
+                directlyProvides(self, iface)
+
+
+    def logPrefix(self):
+        """
+        Transparently pass through the wrapped protocol's log prefix.
+        """
+        if interfaces.ILoggingContext.providedBy(self._wrappedProtocol):
+            return self._wrappedProtocol.logPrefix()
+        return self._wrappedProtocol.__class__.__name__
+
 
     def connectionMade(self):
         """
@@ -72,6 +85,13 @@ class _WrappingProtocol(Protocol):
         return self._wrappedProtocol.dataReceived(data)
 
 
+    def fileDescriptorReceived(self, descriptor):
+        """
+        Proxy C{fileDescriptorReceived} calls to our C{self._wrappedProtocol}
+        """
+        return self._wrappedProtocol.fileDescriptorReceived(descriptor)
+
+
     def connectionLost(self, reason):
         """
         Proxy C{connectionLost} calls to our C{self._wrappedProtocol}
@@ -100,25 +120,72 @@ class _WrappingFactory(ClientFactory):
     """
     Wrap a factory in order to wrap the protocols it builds.
 
-    @ivar _wrappedFactory:  A provider of I{IProtocolFactory} whose
-        buildProtocol method will be called and whose resulting protocol
-        will be wrapped.
+    @ivar _wrappedFactory: A provider of I{IProtocolFactory} whose buildProtocol
+        method will be called and whose resulting protocol will be wrapped.
 
-    @ivar _onConnection: An L{Deferred} that fires when the protocol is
+    @ivar _onConnection: A L{Deferred} that fires when the protocol is
         connected
+
+    @ivar _connector: A L{connector <twisted.internet.interfaces.IConnector>}
+        that is managing the current or previous connection attempt.
     """
     protocol = _WrappingProtocol
 
-    def __init__(self, wrappedFactory, canceller):
+    def __init__(self, wrappedFactory):
         """
         @param wrappedFactory: A provider of I{IProtocolFactory} whose
             buildProtocol method will be called and whose resulting protocol
             will be wrapped.
-        @param canceller: An object that will be called to cancel the
-            L{self._onConnection} L{Deferred}
         """
         self._wrappedFactory = wrappedFactory
-        self._onConnection = defer.Deferred(canceller=canceller)
+        self._onConnection = defer.Deferred(canceller=self._canceller)
+
+
+    def startedConnecting(self, connector):
+        """
+        A connection attempt was started.  Remember the connector which started
+        said attempt, for use later.
+        """
+        self._connector = connector
+
+
+    def _canceller(self, deferred):
+        """
+        The outgoing connection attempt was cancelled.  Fail that L{Deferred}
+        with an L{error.ConnectingCancelledError}.
+
+        @param deferred: The L{Deferred <defer.Deferred>} that was cancelled;
+            should be the same as C{self._onConnection}.
+        @type deferred: L{Deferred <defer.Deferred>}
+
+        @note: This relies on startedConnecting having been called, so it may
+            seem as though there's a race condition where C{_connector} may not
+            have been set.  However, using public APIs, this condition is
+            impossible to catch, because a connection API
+            (C{connectTCP}/C{SSL}/C{UNIX}) is always invoked before a
+            L{_WrappingFactory}'s L{Deferred <defer.Deferred>} is returned to
+            C{connect()}'s caller.
+
+        @return: C{None}
+        """
+        deferred.errback(
+            error.ConnectingCancelledError(
+                self._connector.getDestination()))
+        self._connector.stopConnecting()
+
+
+    def doStart(self):
+        """
+        Start notifications are passed straight through to the wrapped factory.
+        """
+        self._wrappedFactory.doStart()
+
+
+    def doStop(self):
+        """
+        Stop notifications are passed straight through to the wrapped factory.
+        """
+        self._wrappedFactory.doStop()
 
 
     def buildProtocol(self, addr):
@@ -141,37 +208,54 @@ class _WrappingFactory(ClientFactory):
         Errback the C{self._onConnection} L{Deferred} when the
         client connection fails.
         """
-        self._onConnection.errback(reason)
+        if not self._onConnection.called:
+            self._onConnection.errback(reason)
 
 
 
-class TCP4ServerEndpoint(object):
+class StandardIOEndpoint(object):
     """
-    TCP server endpoint with an IPv4 configuration
-
-    @ivar _reactor: An L{IReactorTCP} provider.
-
-    @type _port: int
-    @ivar _port: The port number on which to listen for incoming connections.
-
-    @type _backlog: int
-    @ivar _backlog: size of the listen queue
-
-    @type _interface: str
-    @ivar _interface: the hostname to bind to, defaults to '' (all)
+    A Standard Input/Output endpoint
     """
     implements(interfaces.IStreamServerEndpoint)
 
-    def __init__(self, reactor, port, backlog=50, interface=''):
+    def __init__(self, reactor):
+        """
+        @param reactor: The reactor for the endpoint
+        """
+        self._reactor = reactor
+
+
+    def listen(self, stdioProtocolFactory):
+        """
+        Implement L{IStreamServerEndpoint.listen} to listen on stdin/stdout
+        """
+        return defer.execute(stdio.StandardIO,
+                             stdioProtocolFactory.buildProtocol(PipeAddress()))
+
+
+
+class _TCPServerEndpoint(object):
+    """
+    A TCP server endpoint interface
+    """
+    implements(interfaces.IStreamServerEndpoint)
+
+    def __init__(self, reactor, port, backlog, interface):
         """
         @param reactor: An L{IReactorTCP} provider.
-        @param port: The port number used listening
-        @param backlog: size of the listen queue
-        @param interface: the hostname to bind to, defaults to '' (all)
+
+        @param port: The port number used for listening
+        @type port: int
+
+        @param backlog: Size of the listen queue
+        @type backlog: int
+
+        @param interface: The hostname to bind to
+        @type interface: str
         """
         self._reactor = reactor
         self._port = port
-        self._listenArgs = dict(backlog=50, interface='')
         self._backlog = backlog
         self._interface = interface
 
@@ -188,37 +272,71 @@ class TCP4ServerEndpoint(object):
 
 
 
+class TCP4ServerEndpoint(_TCPServerEndpoint):
+    """
+    Implements TCP server endpoint with an IPv4 configuration
+    """
+    def __init__(self, reactor, port, backlog=50, interface=''):
+        """
+        @param reactor: An L{IReactorTCP} provider.
+
+        @param port: The port number used for listening
+        @type port: int
+
+        @param backlog: Size of the listen queue
+        @type backlog: int
+
+        @param interface: The hostname to bind to, defaults to '' (all)
+        @type interface: str
+        """
+        _TCPServerEndpoint.__init__(self, reactor, port, backlog, interface)
+
+
+
+class TCP6ServerEndpoint(_TCPServerEndpoint):
+    """
+    Implements TCP server endpoint with an IPv6 configuration
+    """
+    def __init__(self, reactor, port, backlog=50, interface='::'):
+        """
+        @param reactor: An L{IReactorTCP} provider.
+
+        @param port: The port number used for listening
+        @type port: int
+
+        @param backlog: Size of the listen queue
+        @type backlog: int
+
+        @param interface: The hostname to bind to, defaults to '' (all)
+        @type interface: str
+        """
+        _TCPServerEndpoint.__init__(self, reactor, port, backlog, interface)
+
+
+
 class TCP4ClientEndpoint(object):
     """
     TCP client endpoint with an IPv4 configuration.
-
-    @ivar _reactor: An L{IReactorTCP} provider.
-
-    @type _host: str
-    @ivar _host: The hostname to connect to as a C{str}
-
-    @type _port: int
-    @ivar _port: The port to connect to as C{int}
-
-    @type _timeout: int
-    @ivar _timeout: number of seconds to wait before assuming the
-        connection has failed.
-
-    @type _bindAddress: tuple
-    @type _bindAddress: a (host, port) tuple of local address to bind
-        to, or None.
     """
     implements(interfaces.IStreamClientEndpoint)
 
     def __init__(self, reactor, host, port, timeout=30, bindAddress=None):
         """
         @param reactor: An L{IReactorTCP} provider
+
         @param host: A hostname, used when connecting
+        @type host: str
+
         @param port: The port number, used when connecting
-        @param timeout: number of seconds to wait before assuming the
+        @type port: int
+
+        @param timeout: The number of seconds to wait before assuming the
             connection has failed.
-        @param bindAddress: a (host, port tuple of local address to bind to,
+        @type timeout: int
+
+        @param bindAddress: A (host, port) tuple of local address to bind to,
             or None.
+        @type bindAddress: tuple
         """
         self._reactor = reactor
         self._host = host
@@ -231,14 +349,9 @@ class TCP4ClientEndpoint(object):
         """
         Implement L{IStreamClientEndpoint.connect} to connect via TCP.
         """
-        def _canceller(deferred):
-            connector.stopConnecting()
-            deferred.errback(
-                error.ConnectingCancelledError(connector.getDestination()))
-
         try:
-            wf = _WrappingFactory(protocolFactory, _canceller)
-            connector = self._reactor.connectTCP(
+            wf = _WrappingFactory(protocolFactory)
+            self._reactor.connectTCP(
                 self._host, self._port, wf,
                 timeout=self._timeout, bindAddress=self._bindAddress)
             return wf._onConnection
@@ -247,27 +360,80 @@ class TCP4ClientEndpoint(object):
 
 
 
+class TCP6ClientEndpoint(object):
+    """
+    TCP client endpoint with an IPv6 configuration.
+
+    @ivar _getaddrinfo: A hook used for testing name resolution.
+
+    @ivar _deferToThread: A hook used for testing deferToThread.
+
+    @ivar _GAI_ADDRESS: Index of the address portion in result of
+        getaddrinfo to be used.
+
+    @ivar _GAI_ADDRESS_HOST: Index of the actual host-address in the
+        5-tuple L{_GAI_ADDRESS}.
+    """
+    _getaddrinfo = socket.getaddrinfo
+    _deferToThread = threads.deferToThread
+    _GAI_ADDRESS = 4
+    _GAI_ADDRESS_HOST = 0
+
+    def __init__(self, reactor, host, port, timeout=30, bindAddress=None):
+        """
+        @param host: An IPv6 address literal or a hostname with an
+            IPv6 address
+
+        @see: L{twisted.internet.interfaces.IReactorTCP.connectTCP}
+        """
+        self._reactor = reactor
+        self._host = host
+        self._port = port
+        self._timeout = timeout
+        self._bindAddress = bindAddress
+
+
+    def connect(self, protocolFactory):
+        """
+        Implement L{IStreamClientEndpoint.connect} to connect via TCP,
+        once the hostname resolution is done.
+        """
+        if isIPv6Address(self._host):
+            d = self._resolvedHostConnect(self._host, protocolFactory)
+        else:
+            d = self._nameResolution(self._host)
+            d.addCallback(lambda result: result[0][self._GAI_ADDRESS]
+                          [self._GAI_ADDRESS_HOST])
+            d.addCallback(self._resolvedHostConnect, protocolFactory)
+        return d
+
+
+    def _nameResolution(self, host):
+        """
+        Resolve the hostname string into a tuple containig the host
+        IPv6 address.
+        """
+        return self._deferToThread(
+            self._getaddrinfo, host, 0, socket.AF_INET6)
+
+
+    def _resolvedHostConnect(self, resolvedHost, protocolFactory):
+        """
+        Connect to the server using the resolved hostname.
+        """
+        try:
+            wf = _WrappingFactory(protocolFactory)
+            self._reactor.connectTCP(resolvedHost, self._port, wf,
+                timeout=self._timeout, bindAddress=self._bindAddress)
+            return wf._onConnection
+        except:
+            return defer.fail()
+
+
+
 class SSL4ServerEndpoint(object):
     """
     SSL secured TCP server endpoint with an IPv4 configuration.
-
-    @ivar _reactor: An L{IReactorSSL} provider.
-
-    @type _host: str
-    @ivar _host: The hostname to connect to as a C{str}
-
-    @type _port: int
-    @ivar _port: The port to connect to as C{int}
-
-    @type _sslContextFactory: L{OpenSSLCertificateOptions}
-    @var _sslContextFactory: SSL Configuration information as an
-        L{OpenSSLCertificateOptions}
-
-    @type _backlog: int
-    @ivar _backlog: size of the listen queue
-
-    @type _interface: str
-    @ivar _interface: the hostname to bind to, defaults to '' (all)
     """
     implements(interfaces.IStreamServerEndpoint)
 
@@ -275,13 +441,18 @@ class SSL4ServerEndpoint(object):
                  backlog=50, interface=''):
         """
         @param reactor: An L{IReactorSSL} provider.
-        @param port: The port number used listening
+
+        @param port: The port number used for listening
+        @type port: int
+
         @param sslContextFactory: An instance of
             L{twisted.internet._sslverify.OpenSSLCertificateOptions}.
-        @param timeout: number of seconds to wait before assuming the
-            connection has failed.
-        @param bindAddress: a (host, port tuple of local address to bind to,
-            or None.
+
+        @param backlog: Size of the listen queue
+        @type backlog: int
+
+        @param interface: The hostname to bind to, defaults to '' (all)
+        @type interface: str
         """
         self._reactor = reactor
         self._port = port
@@ -306,26 +477,6 @@ class SSL4ServerEndpoint(object):
 class SSL4ClientEndpoint(object):
     """
     SSL secured TCP client endpoint with an IPv4 configuration
-
-    @ivar _reactor: An L{IReactorSSL} provider.
-
-    @type _host: str
-    @ivar _host: The hostname to connect to as a C{str}
-
-    @type _port: int
-    @ivar _port: The port to connect to as C{int}
-
-    @type _sslContextFactory: L{OpenSSLCertificateOptions}
-    @var _sslContextFactory: SSL Configuration information as an
-        L{OpenSSLCertificateOptions}
-
-    @type _timeout: int
-    @ivar _timeout: number of seconds to wait before assuming the
-        connection has failed.
-
-    @type _bindAddress: tuple
-    @ivar _bindAddress: a (host, port) tuple of local address to bind
-        to, or None.
     """
     implements(interfaces.IStreamClientEndpoint)
 
@@ -333,14 +484,23 @@ class SSL4ClientEndpoint(object):
                  timeout=30, bindAddress=None):
         """
         @param reactor: An L{IReactorSSL} provider.
+
         @param host: A hostname, used when connecting
+        @type host: str
+
         @param port: The port number, used when connecting
-        @param sslContextFactory: SSL Configuration information as An instance
+        @type port: int
+
+        @param sslContextFactory: SSL Configuration information as an instance
             of L{OpenSSLCertificateOptions}.
-        @param timeout: number of seconds to wait before assuming the
+
+        @param timeout: Number of seconds to wait before assuming the
             connection has failed.
-        @param bindAddress: a (host, port tuple of local address to bind to,
+        @type timeout: int
+
+        @param bindAddress: A (host, port) tuple of local address to bind to,
             or None.
+        @type bindAddress: tuple
         """
         self._reactor = reactor
         self._host = host
@@ -355,14 +515,9 @@ class SSL4ClientEndpoint(object):
         Implement L{IStreamClientEndpoint.connect} to connect with SSL over
         TCP.
         """
-        def _canceller(deferred):
-            connector.stopConnecting()
-            deferred.errback(
-                error.ConnectingCancelledError(connector.getDestination()))
-
         try:
-            wf = _WrappingFactory(protocolFactory, _canceller)
-            connector = self._reactor.connectSSL(
+            wf = _WrappingFactory(protocolFactory)
+            self._reactor.connectSSL(
                 self._host, self._port, wf, self._sslContextFactory,
                 timeout=self._timeout, bindAddress=self._bindAddress)
             return wf._onConnection
@@ -374,15 +529,6 @@ class SSL4ClientEndpoint(object):
 class UNIXServerEndpoint(object):
     """
     UnixSocket server endpoint.
-
-    @type path: str
-    @ivar path: a path to a unix socket on the filesystem.
-
-    @type _listenArgs: dict
-    @ivar _listenArgs: A C{dict} of keyword args that will be passed
-        to L{IReactorUNIX.listenUNIX}
-
-    @var _reactor: An L{IReactorTCP} provider.
     """
     implements(interfaces.IStreamServerEndpoint)
 
@@ -390,13 +536,11 @@ class UNIXServerEndpoint(object):
         """
         @param reactor: An L{IReactorUNIX} provider.
         @param address: The path to the Unix socket file, used when listening
-        @param listenArgs: An optional dict of keyword args that will be
-            passed to L{IReactorUNIX.listenUNIX}
         @param backlog: number of connections to allow in backlog.
         @param mode: mode to set on the unix socket.  This parameter is
             deprecated.  Permissions should be set on the directory which
             contains the UNIX socket.
-        @param wantPID: if True, create a pidfile for the socket.
+        @param wantPID: If True, create a pidfile for the socket.
         """
         self._reactor = reactor
         self._address = address
@@ -420,30 +564,23 @@ class UNIXServerEndpoint(object):
 class UNIXClientEndpoint(object):
     """
     UnixSocket client endpoint.
-
-    @type _path: str
-    @ivar _path: a path to a unix socket on the filesystem.
-
-    @type _timeout: int
-    @ivar _timeout: number of seconds to wait before assuming the connection
-        has failed.
-
-    @type _checkPID: bool
-    @ivar _checkPID: if True, check for a pid file to verify that a server
-        is listening.
-
-    @var _reactor: An L{IReactorUNIX} provider.
     """
     implements(interfaces.IStreamClientEndpoint)
 
     def __init__(self, reactor, path, timeout=30, checkPID=0):
         """
         @param reactor: An L{IReactorUNIX} provider.
+
         @param path: The path to the Unix socket file, used when connecting
-        @param timeout: number of seconds to wait before assuming the
+        @type path: str
+
+        @param timeout: Number of seconds to wait before assuming the
             connection has failed.
-        @param checkPID: if True, check for a pid file to verify that a server
+        @type timeout: int
+
+        @param checkPID: If True, check for a pid file to verify that a server
             is listening.
+        @type checkPID: bool
         """
         self._reactor = reactor
         self._path = path
@@ -456,14 +593,9 @@ class UNIXClientEndpoint(object):
         Implement L{IStreamClientEndpoint.connect} to connect via a
         UNIX Socket
         """
-        def _canceller(deferred):
-            connector.stopConnecting()
-            deferred.errback(
-                error.ConnectingCancelledError(connector.getDestination()))
-
         try:
-            wf = _WrappingFactory(protocolFactory, _canceller)
-            connector = self._reactor.connectUNIX(
+            wf = _WrappingFactory(protocolFactory)
+            self._reactor.connectUNIX(
                 self._path, wf,
                 timeout=self._timeout,
                 checkPID=self._checkPID)
@@ -473,6 +605,53 @@ class UNIXClientEndpoint(object):
 
 
 
+class AdoptedStreamServerEndpoint(object):
+    """
+    An endpoint for listening on a file descriptor initialized outside of
+    Twisted.
+
+    @ivar _used: A C{bool} indicating whether this endpoint has been used to
+        listen with a factory yet.  C{True} if so.
+    """
+    _close = os.close
+    _setNonBlocking = staticmethod(fdesc.setNonBlocking)
+
+    def __init__(self, reactor, fileno, addressFamily):
+        """
+        @param reactor: An L{IReactorSocket} provider.
+
+        @param fileno: An integer file descriptor corresponding to a listening
+            I{SOCK_STREAM} socket.
+
+        @param addressFamily: The address family of the socket given by
+            C{fileno}.
+        """
+        self.reactor = reactor
+        self.fileno = fileno
+        self.addressFamily = addressFamily
+        self._used = False
+
+
+    def listen(self, factory):
+        """
+        Implement L{IStreamServerEndpoint.listen} to start listening on, and
+        then close, C{self._fileno}.
+        """
+        if self._used:
+            return defer.fail(error.AlreadyListened())
+        self._used = True
+
+        try:
+            self._setNonBlocking(self.fileno)
+            port = self.reactor.adoptStreamPort(
+                self.fileno, self.addressFamily, factory)
+            self._close(self.fileno)
+        except:
+            return defer.fail()
+        return defer.succeed(port)
+
+
+
 def _parseTCP(factory, port, interface="", backlog=50):
     """
     Internal parser function for L{_parseServer} to convert the string
@@ -542,7 +721,6 @@ def _parseSSL(factory, port, privateKey="server.pem", certKey=None,
     @param factory: the protocol factory being parsed, or C{None}.  (This was a
         leftover argument from when this code was in C{strports}, and is now
         mostly None and unused.)
-
     @type factory: L{IProtocolFactory} or C{NoneType}
 
     @param port: the integer port number to bind
@@ -577,9 +755,128 @@ def _parseSSL(factory, port, privateKey="server.pem", certKey=None,
     return ((int(port), factory, cf),
             {'interface': interface, 'backlog': int(backlog)})
 
+
+
+class _StandardIOParser(object):
+    """
+    Stream server endpoint string parser for the Standard I/O type.
+
+    @ivar prefix: See L{IStreamClientEndpointStringParser.prefix}.
+    """
+    implements(IPlugin, IStreamServerEndpointStringParser)
+
+    prefix = "stdio"
+
+    def _parseServer(self, reactor):
+        """
+        Internal parser function for L{_parseServer} to convert the string
+        arguments into structured arguments for the L{StandardIOEndpoint}
+
+        @param reactor: Reactor for the endpoint
+        """
+        return StandardIOEndpoint(reactor)
+
+
+    def parseStreamServer(self, reactor, *args, **kwargs):
+        # Redirects to another function (self._parseServer), tricks zope.interface
+        # into believing the interface is correctly implemented.
+        return self._parseServer(reactor)
+
+
+
+class _SystemdParser(object):
+    """
+    Stream server endpoint string parser for the I{systemd} endpoint type.
+
+    @ivar prefix: See L{IStreamClientEndpointStringParser.prefix}.
+
+    @ivar _sddaemon: A L{ListenFDs} instance used to translate an index into an
+        actual file descriptor.
+    """
+    implements(IPlugin, IStreamServerEndpointStringParser)
+
+    _sddaemon = ListenFDs.fromEnvironment()
+
+    prefix = "systemd"
+
+    def _parseServer(self, reactor, domain, index):
+        """
+        Internal parser function for L{_parseServer} to convert the string
+        arguments for a systemd server endpoint into structured arguments for
+        L{AdoptedStreamServerEndpoint}.
+
+        @param reactor: An L{IReactorSocket} provider.
+
+        @param domain: The domain (or address family) of the socket inherited
+            from systemd.  This is a string like C{"INET"} or C{"UNIX"}, ie the
+            name of an address family from the L{socket} module, without the
+            C{"AF_"} prefix.
+        @type domain: C{str}
+
+        @param index: An offset into the list of file descriptors inherited from
+            systemd.
+        @type index: C{str}
+
+        @return: A two-tuple of parsed positional arguments and parsed keyword
+            arguments (a tuple and a dictionary).  These can be used to
+            construct an L{AdoptedStreamServerEndpoint}.
+        """
+        index = int(index)
+        fileno = self._sddaemon.inheritedDescriptors()[index]
+        addressFamily = getattr(socket, 'AF_' + domain)
+        return AdoptedStreamServerEndpoint(reactor, fileno, addressFamily)
+
+
+    def parseStreamServer(self, reactor, *args, **kwargs):
+        # Delegate to another function with a sane signature.  This function has
+        # an insane signature to trick zope.interface into believing the
+        # interface is correctly implemented.
+        return self._parseServer(reactor, *args, **kwargs)
+
+
+
+class _TCP6ServerParser(object):
+    """
+    Stream server endpoint string parser for the TCP6ServerEndpoint type.
+
+    @ivar prefix: See L{IStreamClientEndpointStringParser.prefix}.
+    """
+    implements(IPlugin, IStreamServerEndpointStringParser)
+
+    prefix = "tcp6"     # Used in _parseServer to identify the plugin with the endpoint type
+
+    def _parseServer(self, reactor, port, backlog=50, interface='::'):
+        """
+        Internal parser function for L{_parseServer} to convert the string
+        arguments into structured arguments for the L{TCP6ServerEndpoint}
+
+        @param reactor: An L{IReactorTCP} provider.
+
+        @param port: The port number used for listening
+        @type port: int
+
+        @param backlog: Size of the listen queue
+        @type backlog: int
+
+        @param interface: The hostname to bind to
+        @type interface: str
+        """
+        port = int(port)
+        backlog = int(backlog)
+        return TCP6ServerEndpoint(reactor, port, backlog, interface)
+
+
+    def parseStreamServer(self, reactor, *args, **kwargs):
+        # Redirects to another function (self._parseServer), tricks zope.interface
+        # into believing the interface is correctly implemented.
+        return self._parseServer(reactor, *args, **kwargs)
+
+
+
 _serverParsers = {"tcp": _parseTCP,
                   "unix": _parseUNIX,
-                  "ssl": _parseSSL}
+                  "ssl": _parseSSL,
+                  }
 
 _OP, _STRING = range(2)
 
@@ -703,8 +1000,10 @@ def _parseServer(description, factory, default=None):
     endpointType = args[0]
     parser = _serverParsers.get(endpointType)
     if parser is None:
+        # If the required parser is not found in _server, check if 
+        # a plugin exists for the endpointType
         for plugin in getPlugins(IStreamServerEndpointStringParser):
-            if plugin.prefix == endpointType: 
+            if plugin.prefix == endpointType:
                 return (plugin, args[1:], kw)
         raise ValueError("Unknown endpoint type: '%s'" % (endpointType,))
     return (endpointType.upper(),) + parser(factory, *args[1:], **kw)
@@ -825,16 +1124,32 @@ def quoteStringArgument(argument):
 
 
 
-def _parseClientTCP(**kwargs):
+def _parseClientTCP(*args, **kwargs):
     """
     Perform any argument value coercion necessary for TCP client parameters.
 
+    Valid positional arguments to this function are host and port.
+
     Valid keyword arguments to this function are all L{IReactorTCP.connectTCP}
     arguments.
 
     @return: The coerced values as a C{dict}.
     """
-    kwargs['port'] = int(kwargs['port'])
+
+    if len(args) == 2:
+        kwargs['port'] = int(args[1])
+        kwargs['host'] = args[0]
+    elif len(args) == 1:
+        if 'host' in kwargs:
+            kwargs['port'] = int(args[0])
+        else:
+            kwargs['host'] = args[0]
+
+    try:
+        kwargs['port'] = int(kwargs['port'])
+    except KeyError:
+        pass
+
     try:
         kwargs['timeout'] = int(kwargs['timeout'])
     except KeyError:
@@ -874,7 +1189,7 @@ def _loadCAsFromDir(directoryPath):
 
 
 
-def _parseClientSSL(**kwargs):
+def _parseClientSSL(*args, **kwargs):
     """
     Perform any argument value coercion necessary for SSL client parameters.
 
@@ -883,7 +1198,9 @@ def _parseClientSSL(**kwargs):
     of the certificate file) C{privateKey} (the path name of the private key
     associated with the certificate) are accepted and used to construct a
     context factory.
-    
+
+    Valid positional arguments to this function are host and port.
+
     @param caCertsDir: The one parameter which is not part of
         L{IReactorSSL.connectSSL}'s signature, this is a path name used to
         construct a list of certificate authority certificates.  The directory
@@ -895,7 +1212,7 @@ def _parseClientSSL(**kwargs):
     @return: The coerced values as a C{dict}.
     """
     from twisted.internet import ssl
-    kwargs = _parseClientTCP(**kwargs)
+    kwargs = _parseClientTCP(*args, **kwargs)
     certKey = kwargs.pop('certKey', None)
     privateKey = kwargs.pop('privateKey', None)
     caCertsDir = kwargs.pop('caCertsDir', None)
@@ -926,16 +1243,21 @@ def _parseClientSSL(**kwargs):
 
 
 
-def _parseClientUNIX(**kwargs):
+def _parseClientUNIX(*args, **kwargs):
     """
     Perform any argument value coercion necessary for UNIX client parameters.
 
     Valid keyword arguments to this function are all L{IReactorUNIX.connectUNIX}
-    arguments except for C{checkPID}.  Instead, C{lockfile} is accepted and has
-    the same meaning.
+    keyword arguments except for C{checkPID}.  Instead, C{lockfile} is accepted
+    and has the same meaning.  Also C{path} is used instead of C{address}.
+    
+    Valid positional arguments to this function are C{path}.
 
     @return: The coerced values as a C{dict}.
     """
+    if len(args) == 1:
+        kwargs['path'] = args[0]
+
     try:
         kwargs['checkPID'] = bool(int(kwargs.pop('lockfile')))
     except KeyError:
@@ -959,19 +1281,27 @@ def clientFromString(reactor, description):
     Construct a client endpoint from a description string.
 
     Client description strings are much like server description strings,
-    although they take all of their arguments as keywords, since even the
-    simplest client endpoint (plain TCP) requires at least 2 arguments (host
-    and port) to construct.
+    although they take all of their arguments as keywords, aside from host and
+    port.
 
     You can create a TCP client endpoint with the 'host' and 'port' arguments,
     like so::
 
         clientFromString(reactor, "tcp:host=www.example.com:port=80")
 
+    or, without specifying host and port keywords::
+
+        clientFromString(reactor, "tcp:www.example.com:80")
+
+    Or you can specify only one or the other, as in the following 2 examples::
+
+        clientFromString(reactor, "tcp:host=www.example.com:80")
+        clientFromString(reactor, "tcp:www.example.com:port=80")
+
     or an SSL client endpoint with those arguments, plus the arguments used by
     the server SSL, for a client certificate::
 
-        clientFromString(reactor, "ssl:host=web.example.com:port=443:"
+        clientFromString(reactor, "ssl:web.example.com:443:"
                                   "privateKey=foo.pem:certKey=foo.pem")
 
     to specify your certificate trust roots, you can identify a directory with
@@ -979,6 +1309,17 @@ def clientFromString(reactor, description):
 
         clientFromString(reactor, "ssl:host=web.example.com:port=443:"
                                   "caCertsDir=/etc/ssl/certs")
+    
+    You can create a UNIX client endpoint with the 'path' argument and optional
+    'lockfile' and 'timeout' arguments::
+    
+        clientFromString(reactor, "unix:path=/var/foo/bar:lockfile=1:timeout=9")
+    
+    or, with the path as a positional argument with or without optional
+    arguments as in the following 2 examples::
+    
+        clientFromString(reactor, "unix:/var/foo/bar")
+        clientFromString(reactor, "unix:/var/foo/bar:lockfile=1:timeout=9")
 
     This function is also extensible; new endpoint types may be registered as
     L{IStreamClientEndpointStringParser} plugins.  See that interface for more
