@@ -3,8 +3,6 @@
 
 """
 Exceptions and errors for use in twisted.internet modules.
-
-Maintainer: Itamar Shtull-Trauring
 """
 
 import socket
@@ -97,8 +95,8 @@ class ConnectError(Exception):
         s = self.__doc__ or self.__class__.__name__
         if self.osError:
             s = '%s: %s' % (s, self.osError)
-        if self[0]:
-            s = '%s: %s' % (s, self[0])
+        if self.args[0]:
+            s = '%s: %s' % (s, self.args[0])
         s = '%s.' % s
         return s
 
@@ -223,6 +221,16 @@ class ConnectionLost(ConnectionClosed):
 
 
 
+class ConnectionAborted(ConnectionLost):
+    """
+    Connection was aborted locally, using
+    L{twisted.internet.interfaces.ITCPTransport.abortConnection}.
+
+    @since: 11.1
+    """
+
+
+
 class ConnectionDone(ConnectionClosed):
     """Connection was closed cleanly"""
 
@@ -235,6 +243,19 @@ class ConnectionDone(ConnectionClosed):
 
 
 
+class FileDescriptorOverrun(ConnectionLost):
+    """
+    A mis-use of L{IUNIXTransport.sendFileDescriptor} caused the connection to
+    be closed.
+
+    Each file descriptor sent using C{sendFileDescriptor} must be associated
+    with at least one byte sent using L{ITransport.write}.  If at any point
+    fewer bytes have been written than file descriptors have been sent, the
+    connection is closed with this exception.
+    """
+
+
+
 class ConnectionFdescWentAway(ConnectionLost):
     """Uh""" #TODO
 
@@ -389,6 +410,28 @@ class ConnectingCancelledError(Exception):
 
 
 
+class UnsupportedAddressFamily(Exception):
+    """
+    An attempt was made to use a socket with an address family (eg I{AF_INET},
+    I{AF_INET6}, etc) which is not supported by the reactor.
+    """
+
+
+
+class UnsupportedSocketType(Exception):
+    """
+    An attempt was made to use a socket of a type (eg I{SOCK_STREAM},
+    I{SOCK_DGRAM}, etc) which is not supported by the reactor.
+    """
+
+
+class AlreadyListened(Exception):
+    """
+    An attempt was made to listen on a file descriptor which can only be
+    listened on once.
+    """
+
+
 __all__ = [
     'BindError', 'CannotListenError', 'MulticastJoinError',
     'MessageLengthError', 'DNSLookupError', 'ConnectInProgressError',
@@ -401,4 +444,5 @@ __all__ = [
     'AlreadyCancelled', 'PotentialZombieWarning', 'ProcessDone',
     'ProcessTerminated', 'ProcessExitedAlready', 'NotConnectingError',
     'NotListeningError', 'ReactorNotRunning', 'ReactorAlreadyRunning',
-    'ReactorAlreadyInstalledError', 'ConnectingCancelledError']
+    'ReactorAlreadyInstalledError', 'ConnectingCancelledError',
+    'UnsupportedAddressFamily', 'UnsupportedSocketType']
