@@ -8,7 +8,7 @@ import sys, os, random, gc, time, warnings
 
 from twisted.internet import defer
 from twisted.application import app
-from twisted.python import usage, reflect, failure, versions, deprecate
+from twisted.python import usage, reflect, failure
 from twisted.python.filepath import FilePath
 from twisted import plugin
 from twisted.python.util import spewer
@@ -89,8 +89,9 @@ def isTestFile(filename):
             and os.path.splitext(basename)[1] == ('.py'))
 
 
-def _zshReporterAction():
-    return "(%s)" % (" ".join([p.longOpt for p in plugin.getPlugins(itrial.IReporter)]),)
+def _reporterAction():
+    return usage.CompleteList([p.longOpt for p in
+                               plugin.getPlugins(itrial.IReporter)])
 
 class Options(usage.Options, app.ReactorSelectionMixin):
     synopsis = """%s [options] [[file|package|module|TestCase|testmethod]...]
@@ -130,14 +131,17 @@ class Options(usage.Options, app.ReactorSelectionMixin):
          'The reporter to use for this test run.  See --help-reporters for '
          'more info.']]
 
-    zsh_actions = {"tbformat":"(plain emacs cgitb)",
-                   "reporter":_zshReporterAction}
-    zsh_actionDescr = {"logfile":"log file name",
-                       "random":"random seed"}
-    zsh_extras = ["*:file|module|package|TestCase|testMethod:_files -g '*.py'"]
+    compData = usage.Completions(
+        optActions={"tbformat": usage.CompleteList(["plain", "emacs", "cgitb"]),
+                    "reporter": _reporterAction,
+                    "logfile": usage.CompleteFiles(descr="log file name"),
+                    "random": usage.Completer(descr="random seed")},
+        extraActions=[usage.CompleteFiles(
+                "*.py", descr="file | module | package | TestCase | testMethod",
+                repeat=True)],
+        )
 
     fallbackReporter = reporter.TreeReporter
-    extra = None
     tracer = None
 
     def __init__(self):
@@ -158,8 +162,8 @@ class Options(usage.Options, app.ReactorSelectionMixin):
 
     def opt_coverage(self):
         """
-        Generate coverage information in the I{coverage} file in the
-        directory specified by the I{trial-temp} option.
+        Generate coverage information in the coverage file in the
+        directory specified by the temp-directory option.
         """
         import trace
         self.tracer = trace.Trace(count=1, trace=0)
@@ -229,21 +233,6 @@ class Options(usage.Options, app.ReactorSelectionMixin):
                 "tbformat must be 'plain', 'emacs', or 'cgitb'.")
 
 
-    def opt_extra(self, arg):
-        """
-        Add an extra argument.  (This is a hack necessary for interfacing with
-        emacs's `gud'.)  NOTE: This option is deprecated as of Twisted 11.0
-        """
-        warnings.warn(deprecate.getDeprecationWarningString(Options.opt_extra,
-                                                            versions.Version('Twisted', 11, 0, 0)),
-                      category=DeprecationWarning, stacklevel=2)
-
-        if self.extra is None:
-            self.extra = []
-        self.extra.append(arg)
-    opt_x = opt_extra
-
-
     def opt_recursionlimit(self, arg):
         """
         see sys.setrecursionlimit()
@@ -283,8 +272,6 @@ class Options(usage.Options, app.ReactorSelectionMixin):
 
     def parseArgs(self, *args):
         self['tests'].update(args)
-        if self.extra is not None:
-            self['tests'].update(self.extra)
 
 
     def _loadReporterByName(self, name):
