@@ -4,10 +4,13 @@
 """
 Test cases for failure module.
 """
+from __future__ import division
 
+import re
 import sys
 import StringIO
 import traceback
+import pdb
 
 from twisted.trial import unittest, util
 
@@ -19,11 +22,17 @@ except ImportError:
     raiser = None
 
 
-def getDivisionFailure():
+def getDivisionFailure(*args, **kwargs):
+    """
+    Make a C{Failure} of a divide-by-zero error.
+
+    @param args: Any C{*args} are passed to Failure's constructor.
+    @param kwargs: Any C{**kwargs} are passed to Failure's constructor.
+    """
     try:
         1/0
     except:
-        f = failure.Failure()
+        f = failure.Failure(*args, **kwargs)
     return f
 
 
@@ -36,8 +45,9 @@ class FailureTestCase(unittest.TestCase):
         except:
             f = failure.Failure()
         error = f.trap(SystemExit, RuntimeError)
-        self.assertEquals(error, RuntimeError)
-        self.assertEquals(f.type, NotImplementedError)
+        self.assertEqual(error, RuntimeError)
+        self.assertEqual(f.type, NotImplementedError)
+
 
     def test_notTrapped(self):
         """Making sure trap doesn't trap what it shouldn't."""
@@ -47,21 +57,104 @@ class FailureTestCase(unittest.TestCase):
             f = failure.Failure()
         self.assertRaises(failure.Failure, f.trap, OverflowError)
 
-    def testPrinting(self):
+
+    def assertStartsWith(self, s, prefix):
+        """
+        Assert that s starts with a particular prefix.
+        """
+        self.assertTrue(s.startswith(prefix),
+                        '%r is not the start of %r' % (prefix, s))
+
+
+    def test_printingSmokeTest(self):
+        """
+        None of the print* methods fail when called.
+        """
+        f = getDivisionFailure()
         out = StringIO.StringIO()
-        try:
-            1/0
-        except:
-            f = failure.Failure()
         f.printDetailedTraceback(out)
+        self.assertStartsWith(out.getvalue(), '*--- Failure')
+        out = StringIO.StringIO()
         f.printBriefTraceback(out)
+        self.assertStartsWith(out.getvalue(), 'Traceback')
+        out = StringIO.StringIO()
         f.printTraceback(out)
+        self.assertStartsWith(out.getvalue(), 'Traceback')
+
+
+    def test_printingCapturedVarsSmokeTest(self):
+        """
+        None of the print* methods fail when called on a L{Failure} constructed
+        with C{captureVars=True}.
+
+        Local variables on the stack can be seen in the detailed traceback.
+        """
+        exampleLocalVar = 'xyzzy'
+        f = getDivisionFailure(captureVars=True)
+        out = StringIO.StringIO()
+        f.printDetailedTraceback(out)
+        self.assertStartsWith(out.getvalue(), '*--- Failure')
+        self.assertNotEqual(None, re.search('exampleLocalVar.*xyzzy',
+                                            out.getvalue()))
+        out = StringIO.StringIO()
+        f.printBriefTraceback(out)
+        self.assertStartsWith(out.getvalue(), 'Traceback')
+        out = StringIO.StringIO()
+        f.printTraceback(out)
+        self.assertStartsWith(out.getvalue(), 'Traceback')
+
+
+    def test_printingCapturedVarsCleanedSmokeTest(self):
+        """
+        C{printDetailedTraceback} includes information about local variables on
+        the stack after C{cleanFailure} has been called.
+        """
+        exampleLocalVar = 'xyzzy'
+        f = getDivisionFailure(captureVars=True)
+        f.cleanFailure()
+        out = StringIO.StringIO()
+        f.printDetailedTraceback(out)
+        self.assertNotEqual(None, re.search('exampleLocalVar.*xyzzy',
+                                            out.getvalue()))
+
+
+    def test_printingNoVars(self):
+        """
+        Calling C{Failure()} with no arguments does not capture any locals or
+        globals, so L{printDetailedTraceback} cannot show them in its output.
+        """
+        out = StringIO.StringIO()
+        f = getDivisionFailure()
+        f.printDetailedTraceback(out)
+        # There should be no variables in the detailed output.  Variables are
+        # printed on lines with 2 leading spaces.
+        linesWithVars = [line for line in out.getvalue().splitlines()
+                         if line.startswith('  ')]
+        self.assertEqual([], linesWithVars)
+        self.assertSubstring(
+            'Capture of Locals and Globals disabled', out.getvalue())
+
+
+    def test_printingCaptureVars(self):
+        """
+        Calling C{Failure(captureVars=True)} captures the locals and globals
+        for its stack frames, so L{printDetailedTraceback} will show them in
+        its output.
+        """
+        out = StringIO.StringIO()
+        f = getDivisionFailure(captureVars=True)
+        f.printDetailedTraceback(out)
+        # Variables are printed on lines with 2 leading spaces.
+        linesWithVars = [line for line in out.getvalue().splitlines()
+                         if line.startswith('  ')]
+        self.assertNotEqual([], linesWithVars)
+
 
     def testExplictPass(self):
         e = RuntimeError()
         f = failure.Failure(e)
         f.trap(RuntimeError)
-        self.assertEquals(f.value, e)
+        self.assertEqual(f.value, e)
 
 
     def _getInnermostFrameLine(self, f):
@@ -78,14 +171,14 @@ class FailureTestCase(unittest.TestCase):
     def testRaiseExceptionWithTB(self):
         f = getDivisionFailure()
         innerline = self._getInnermostFrameLine(f)
-        self.assertEquals(innerline, '1/0')
+        self.assertEqual(innerline, '1/0')
 
 
     def testLackOfTB(self):
         f = getDivisionFailure()
         f.cleanFailure()
         innerline = self._getInnermostFrameLine(f)
-        self.assertEquals(innerline, '1/0')
+        self.assertEqual(innerline, '1/0')
 
     testLackOfTB.todo = "the traceback is not preserved, exarkun said he'll try to fix this! god knows how"
 
@@ -98,13 +191,14 @@ class FailureTestCase(unittest.TestCase):
             f = failure.Failure()
         return f
 
+
     def test_raiseStringExceptions(self):
         # String exceptions used to totally bugged f.raiseException
         f = self._getStringFailure()
         try:
             f.raiseException()
         except:
-            self.assertEquals(sys.exc_info()[0], self._stringException)
+            self.assertEqual(sys.exc_info()[0], self._stringException)
         else:
             raise AssertionError("Should have raised")
     test_raiseStringExceptions.suppress = [
@@ -141,31 +235,59 @@ class FailureTestCase(unittest.TestCase):
         """
         self.assertRaises(failure.NoCurrentExceptionError, failure.Failure)
 
+
     def test_getTracebackObject(self):
         """
         If the C{Failure} has not been cleaned, then C{getTracebackObject}
-        should return the traceback object that it was given in the
-        constructor.
+        returns the traceback object that captured in its constructor.
         """
         f = getDivisionFailure()
         self.assertEqual(f.getTracebackObject(), f.tb)
 
+
+    def test_getTracebackObjectFromCaptureVars(self):
+        """
+        C{captureVars=True} has no effect on the result of
+        C{getTracebackObject}.
+        """
+        try:
+            1/0
+        except ZeroDivisionError:
+            noVarsFailure = failure.Failure()
+            varsFailure = failure.Failure(captureVars=True)
+        self.assertEqual(noVarsFailure.getTracebackObject(), varsFailure.tb)
+
+
     def test_getTracebackObjectFromClean(self):
         """
-        If the Failure has been cleaned, then C{getTracebackObject} should
-        return an object that looks the same to L{traceback.extract_tb}.
+        If the Failure has been cleaned, then C{getTracebackObject} returns an
+        object that looks the same to L{traceback.extract_tb}.
         """
         f = getDivisionFailure()
         expected = traceback.extract_tb(f.getTracebackObject())
         f.cleanFailure()
         observed = traceback.extract_tb(f.getTracebackObject())
+        self.assertNotEqual(None, expected)
         self.assertEqual(expected, observed)
 
+
+    def test_getTracebackObjectFromCaptureVarsAndClean(self):
+        """
+        If the Failure was created with captureVars, then C{getTracebackObject}
+        returns an object that looks the same to L{traceback.extract_tb}.
+        """
+        f = getDivisionFailure(captureVars=True)
+        expected = traceback.extract_tb(f.getTracebackObject())
+        f.cleanFailure()
+        observed = traceback.extract_tb(f.getTracebackObject())
+        self.assertEqual(expected, observed)
+
+
     def test_getTracebackObjectWithoutTraceback(self):
         """
         L{failure.Failure}s need not be constructed with traceback objects. If
         a C{Failure} has no traceback information at all, C{getTracebackObject}
-        should just return None.
+        just returns None.
 
         None is a good value, because traceback.extract_tb(None) -> [].
         """
@@ -383,6 +505,7 @@ class TestFormattableTraceback(unittest.TestCase):
         self.assertEqual(traceback.extract_tb(tb),
                          [('filename.py', 123, 'method', None)])
 
+
     def test_manyFrames(self):
         """
         A C{_Traceback} object constructed with multiple frames should be able
@@ -416,5 +539,57 @@ class TestFrameAttributes(unittest.TestCase):
         self.assertIsInstance(frame.f_code, failure._Code)
 
 
+
+class TestDebugMode(unittest.TestCase):
+    """
+    Failure's debug mode should allow jumping into the debugger.
+    """
+
+    def setUp(self):
+        """
+        Override pdb.post_mortem so we can make sure it's called.
+        """
+        # Make sure any changes we make are reversed:
+        post_mortem = pdb.post_mortem
+        origInit = failure.Failure.__dict__['__init__']
+        def restore():
+            pdb.post_mortem = post_mortem
+            failure.Failure.__dict__['__init__'] = origInit
+        self.addCleanup(restore)
+
+        self.result = []
+        pdb.post_mortem = self.result.append
+        failure.startDebugMode()
+
+
+    def test_regularFailure(self):
+        """
+        If startDebugMode() is called, calling Failure() will first call
+        pdb.post_mortem with the traceback.
+        """
+        try:
+            1/0
+        except:
+            typ, exc, tb = sys.exc_info()
+            f = failure.Failure()
+        self.assertEqual(self.result, [tb])
+        self.assertEqual(f.captureVars, False)
+
+
+    def test_captureVars(self):
+        """
+        If startDebugMode() is called, passing captureVars to Failure() will
+        not blow up.
+        """
+        try:
+            1/0
+        except:
+            typ, exc, tb = sys.exc_info()
+            f = failure.Failure(captureVars=True)
+        self.assertEqual(self.result, [tb])
+        self.assertEqual(f.captureVars, True)
+
+
+
 if sys.version_info[:2] >= (2, 5):
     from twisted.test.generator_failure_tests import TwoPointFiveFailureTests
