@@ -1,4 +1,4 @@
-# Copyright (c) 2001-2010 Twisted Matrix Laboratories.
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
@@ -15,12 +15,15 @@ from twisted.python.runtime import platform
 
 from twisted.trial import unittest
 
+from zope.interface.verify import verifyObject
+
 
 class AbstractFilePathTestCase(unittest.TestCase):
 
     f1content = "file 1"
     f2content = "file 2"
 
+
     def _mkpath(self, *p):
         x = os.path.abspath(os.path.join(self.cmn, *p))
         self.all.append(x)
@@ -62,12 +65,13 @@ class AbstractFilePathTestCase(unittest.TestCase):
         """
         Verify that the segments between two paths are correctly identified.
         """
-        self.assertEquals(
+        self.assertEqual(
             self.path.child("a").child("b").child("c").segmentsFrom(self.path),
             ["a", "b", "c"])
 
     def test_segmentsFromNegative(self):
-        """Verify that segmentsFrom notices when the ancestor isn't an ancestor.
+        """
+        Verify that segmentsFrom notices when the ancestor isn't an ancestor.
         """
         self.assertRaises(
             ValueError,
@@ -81,7 +85,7 @@ class AbstractFilePathTestCase(unittest.TestCase):
         hierarchy.
         """
         x = [foo.path for foo in self.path.walk()]
-        self.assertEquals(set(x), set(self.all))
+        self.assertEqual(set(x), set(self.all))
 
 
     def test_parents(self):
@@ -98,11 +102,12 @@ class AbstractFilePathTestCase(unittest.TestCase):
             L.append(thispath)
             lastpath = thispath
             thispath = os.path.dirname(thispath)
-        self.assertEquals([x.path for x in pathobj.parents()], L)
+        self.assertEqual([x.path for x in pathobj.parents()], L)
 
 
     def test_validSubdir(self):
-        """Verify that a valid subdirectory will show up as a directory, but not as a
+        """
+        Verify that a valid subdirectory will show up as a directory, but not as a
         file, not as a symlink, and be listable.
         """
         sub1 = self.path.child('sub1')
@@ -114,7 +119,7 @@ class AbstractFilePathTestCase(unittest.TestCase):
                         "It's a directory.")
         self.failUnless(not sub1.islink(),
                         "It's a directory.")
-        self.failUnlessEqual(sub1.listdir(),
+        self.assertEqual(sub1.listdir(),
                              ['file2'])
 
 
@@ -131,9 +136,9 @@ class AbstractFilePathTestCase(unittest.TestCase):
         Make sure that we can read existent non-empty files.
         """
         f1 = self.path.child('file1')
-        self.failUnlessEqual(f1.open().read(), self.f1content)
+        self.assertEqual(f1.open().read(), self.f1content)
         f2 = self.path.child('sub1').child('file2')
-        self.failUnlessEqual(f2.open().read(), self.f2content)
+        self.assertEqual(f2.open().read(), self.f2content)
 
 
     def test_multipleChildSegments(self):
@@ -143,7 +148,7 @@ class AbstractFilePathTestCase(unittest.TestCase):
         """
         multiple = self.path.descendant(['a', 'b', 'c'])
         single = self.path.child('a').child('b').child('c')
-        self.assertEquals(multiple, single)
+        self.assertEqual(multiple, single)
 
 
     def test_dictionaryKeys(self):
@@ -156,13 +161,13 @@ class AbstractFilePathTestCase(unittest.TestCase):
         dictoid = {}
         dictoid[f1] = 3
         dictoid[f1prime] = 4
-        self.assertEquals(dictoid[f1], 4)
-        self.assertEquals(dictoid.keys(), [f1])
+        self.assertEqual(dictoid[f1], 4)
+        self.assertEqual(dictoid.keys(), [f1])
         self.assertIdentical(dictoid.keys()[0], f1)
         self.assertNotIdentical(dictoid.keys()[0], f1prime) # sanity check
         dictoid[f2] = 5
-        self.assertEquals(dictoid[f2], 5)
-        self.assertEquals(len(dictoid), 2)
+        self.assertEqual(dictoid[f2], 5)
+        self.assertEqual(len(dictoid), 2)
 
 
     def test_dictionaryKeyWithString(self):
@@ -173,7 +178,7 @@ class AbstractFilePathTestCase(unittest.TestCase):
         f1 = self.path.child('file1')
         dictoid = {f1: 'hello'}
         dictoid[f1.path] = 'goodbye'
-        self.assertEquals(len(dictoid), 2)
+        self.assertEqual(len(dictoid), 2)
 
 
     def test_childrenNonexistentError(self):
@@ -199,9 +204,9 @@ class AbstractFilePathTestCase(unittest.TestCase):
         (and hopefully therefore 'high precision').
         """
         for p in self.path, self.path.child('file1'):
-            self.failUnlessEqual(type(p.getAccessTime()), float)
-            self.failUnlessEqual(type(p.getModificationTime()), float)
-            self.failUnlessEqual(type(p.getStatusChangeTime()), float)
+            self.assertEqual(type(p.getAccessTime()), float)
+            self.assertEqual(type(p.getModificationTime()), float)
+            self.assertEqual(type(p.getStatusChangeTime()), float)
 
 
     def test_oldTimesAreInts(self):
@@ -210,9 +215,9 @@ class AbstractFilePathTestCase(unittest.TestCase):
         integers, for compatibility.
         """
         for p in self.path, self.path.child('file1'):
-            self.failUnlessEqual(type(p.getatime()), int)
-            self.failUnlessEqual(type(p.getmtime()), int)
-            self.failUnlessEqual(type(p.getctime()), int)
+            self.assertEqual(type(p.getatime()), int)
+            self.assertEqual(type(p.getmtime()), int)
+            self.assertEqual(type(p.getctime()), int)
 
 
 
@@ -266,13 +271,13 @@ class ListingCompatibilityTests(unittest.TestCase):
         d2 = ose.originalException.__dict__.keys()
         d1.sort()
         d2.sort()
-        self.assertEquals(d1, d2)
+        self.assertEqual(d1, d2)
 
 
 
 def zipit(dirname, zfname):
     """
-    create a zipfile on zfname, containing the contents of dirname'
+    Create a zipfile on zfname, containing the contents of dirname'
     """
     zf = zipfile.ZipFile(zfname, "w")
     for root, ignored, files, in os.walk(dirname):
@@ -298,6 +303,14 @@ class ZipFilePathTestCase(AbstractFilePathTestCase):
         self.all = [x.replace(self.cmn, self.cmn + '.zip') for x in self.all]
 
 
+    def test_verifyObject(self):
+        """
+        ZipPaths implement IFilePath.
+        """
+
+        self.assertTrue(verifyObject(filepath.IFilePath, self.path))
+
+
     def test_zipPathRepr(self):
         """
         Make sure that invoking ZipPath's repr prints the correct class name
@@ -308,7 +321,7 @@ class ZipFilePathTestCase(AbstractFilePathTestCase):
             os.path.abspath(self.cmn + ".zip" + os.sep + 'foo'),)
 
         # Check for an absolute path
-        self.assertEquals(repr(child), pathRepr)
+        self.assertEqual(repr(child), pathRepr)
 
         # Create a path to the file rooted in the current working directory
         relativeCommon = self.cmn.replace(os.getcwd() + os.sep, "", 1) + ".zip"
@@ -316,7 +329,7 @@ class ZipFilePathTestCase(AbstractFilePathTestCase):
         child = relpath.child("foo")
 
         # Check using a path without the cwd prepended
-        self.assertEquals(repr(child), pathRepr)
+        self.assertEqual(repr(child), pathRepr)
 
 
     def test_zipPathReprParentDirSegment(self):
@@ -328,7 +341,7 @@ class ZipFilePathTestCase(AbstractFilePathTestCase):
         child = self.path.child("foo").child("..").child("bar")
         pathRepr = "ZipPath(%r)" % (
             self.cmn + ".zip" + os.sep.join(["", "foo", "..", "bar"]))
-        self.assertEquals(repr(child), pathRepr)
+        self.assertEqual(repr(child), pathRepr)
 
 
     def test_zipPathReprEscaping(self):
@@ -339,7 +352,7 @@ class ZipFilePathTestCase(AbstractFilePathTestCase):
         child = self.path.child("'")
         path = self.cmn + ".zip" + os.sep.join(["", "'"])
         pathRepr = "ZipPath('%s')" % (path.encode('string-escape'),)
-        self.assertEquals(repr(child), pathRepr)
+        self.assertEqual(repr(child), pathRepr)
 
 
     def test_zipArchiveRepr(self):
@@ -350,14 +363,14 @@ class ZipFilePathTestCase(AbstractFilePathTestCase):
         pathRepr = 'ZipArchive(%r)' % (os.path.abspath(self.cmn + '.zip'),)
 
         # Check for an absolute path
-        self.assertEquals(repr(self.path), pathRepr)
+        self.assertEqual(repr(self.path), pathRepr)
 
         # Create a path to the file rooted in the current working directory
         relativeCommon = self.cmn.replace(os.getcwd() + os.sep, "", 1) + ".zip"
         relpath = ZipArchive(relativeCommon)
 
         # Check using a path without the cwd prepended
-        self.assertEquals(repr(relpath), pathRepr)
+        self.assertEqual(repr(relpath), pathRepr)
 
 
 
@@ -490,20 +503,176 @@ class ExplodingFilePath(filepath.FilePath):
 
 
 
+class PermissionsTestCase(unittest.TestCase):
+    """
+    Test Permissions and RWX classes
+    """
+
+    def assertNotUnequal(self, first, second, msg=None):
+        """
+        Tests that C{first} != C{second} is false.  This method tests the
+        __ne__ method, as opposed to L{assertEqual} (C{first} == C{second}),
+        which tests the __eq__ method.
+
+        Note: this should really be part of trial
+        """
+        if first != second:
+            if msg is None:
+                msg = '';
+            if len(msg) > 0:
+                msg += '\n'
+            raise self.failureException(
+                '%snot not unequal (__ne__ not implemented correctly):'
+                '\na = %s\nb = %s\n'
+                % (msg, pformat(first), pformat(second)))
+        return first
+
+
+    def test_rwxFromBools(self):
+        """
+        L{RWX}'s constructor takes a set of booleans
+        """
+        for r in (True, False):
+            for w in (True, False):
+                for x in (True, False):
+                    rwx = filepath.RWX(r, w, x)
+                    self.assertEqual(rwx.read, r)
+                    self.assertEqual(rwx.write, w)
+                    self.assertEqual(rwx.execute, x)
+        rwx = filepath.RWX(True, True, True)
+        self.assertTrue(rwx.read and rwx.write and rwx.execute)
+
+
+    def test_rwxEqNe(self):
+        """
+        L{RWX}'s created with the same booleans are equivalent.  If booleans
+        are different, they are not equal.
+        """
+        for r in (True, False):
+            for w in (True, False):
+                for x in (True, False):
+                    self.assertEqual(filepath.RWX(r, w, x),
+                                      filepath.RWX(r, w, x))
+                    self.assertNotUnequal(filepath.RWX(r, w, x),
+                                          filepath.RWX(r, w, x))
+        self.assertNotEqual(filepath.RWX(True, True, True),
+                            filepath.RWX(True, True, False))
+        self.assertNotEqual(3, filepath.RWX(True, True, True))
+
+
+    def test_rwxShorthand(self):
+        """
+        L{RWX}'s shorthand string should be 'rwx' if read, write, and execute
+        permission bits are true.  If any of those permissions bits are false,
+        the character is replaced by a '-'.
+        """
+
+        def getChar(val, letter):
+            if val:
+                return letter
+            return '-'
+
+        for r in (True, False):
+            for w in (True, False):
+                for x in (True, False):
+                    rwx = filepath.RWX(r, w, x)
+                    self.assertEqual(rwx.shorthand(),
+                                      getChar(r, 'r') +
+                                      getChar(w, 'w') +
+                                      getChar(x, 'x'))
+        self.assertEqual(filepath.RWX(True, False, True).shorthand(), "r-x")
+
+
+    def test_permissionsFromStat(self):
+        """
+        L{Permissions}'s constructor takes a valid permissions bitmask and
+        parsaes it to produce the correct set of boolean permissions.
+        """
+        def _rwxFromStat(statModeInt, who):
+            def getPermissionBit(what, who):
+                return (statModeInt &
+                        getattr(stat, "S_I%s%s" % (what, who))) > 0
+            return filepath.RWX(*[getPermissionBit(what, who) for what in
+                         ('R', 'W', 'X')])
+
+        for u in range(0, 8):
+            for g in range(0, 8):
+                for o in range(0, 8):
+                    chmodString = "%d%d%d" % (u, g, o)
+                    chmodVal = int(chmodString, 8)
+                    perm = filepath.Permissions(chmodVal)
+                    self.assertEqual(perm.user,
+                                      _rwxFromStat(chmodVal, "USR"),
+                                      "%s: got user: %s" %
+                                      (chmodString, perm.user))
+                    self.assertEqual(perm.group,
+                                      _rwxFromStat(chmodVal, "GRP"),
+                                      "%s: got group: %s" %
+                                      (chmodString, perm.group))
+                    self.assertEqual(perm.other,
+                                      _rwxFromStat(chmodVal, "OTH"),
+                                      "%s: got other: %s" %
+                                      (chmodString, perm.other))
+        perm = filepath.Permissions(0777)
+        for who in ("user", "group", "other"):
+            for what in ("read", "write", "execute"):
+                self.assertTrue(getattr(getattr(perm, who), what))
+
+
+    def test_permissionsEq(self):
+        """
+        Two L{Permissions}'s that are created with the same bitmask
+        are equivalent
+        """
+        self.assertEqual(filepath.Permissions(0777),
+                          filepath.Permissions(0777))
+        self.assertNotUnequal(filepath.Permissions(0777),
+                              filepath.Permissions(0777))
+        self.assertNotEqual(filepath.Permissions(0777),
+                            filepath.Permissions(0700))
+        self.assertNotEqual(3, filepath.Permissions(0777))
+
+
+    def test_permissionsShorthand(self):
+        """
+        L{Permissions}'s shorthand string is the RWX shorthand string for its
+        user permission bits, group permission bits, and other permission bits
+        concatenated together, without a space.
+        """
+        for u in range(0, 8):
+            for g in range(0, 8):
+                for o in range(0, 8):
+                    perm = filepath.Permissions(eval("0%d%d%d" % (u, g, o)))
+                    self.assertEqual(perm.shorthand(),
+                                      ''.join(x.shorthand() for x in (
+                                          perm.user, perm.group, perm.other)))
+        self.assertEqual(filepath.Permissions(0770).shorthand(), "rwxrwx---")
+
+
+
 class FilePathTestCase(AbstractFilePathTestCase):
     """
     Test various L{FilePath} path manipulations.
     """
 
+
+    def test_verifyObject(self):
+        """
+        FilePaths implement IFilePath.
+        """
+
+        self.assertTrue(verifyObject(filepath.IFilePath, self.path))
+
+
     def test_chmod(self):
         """
-        Make sure that calling L{FilePath.chmod} modifies the permissions of
+        L{FilePath.chmod} modifies the permissions of
         the passed file as expected (using C{os.stat} to check). We use some
         basic modes that should work everywhere (even on Windows).
         """
         for mode in (0555, 0777):
             self.path.child("sub1").chmod(mode)
-            self.assertEquals(
+            self.assertEqual(
                 stat.S_IMODE(os.stat(self.path.child("sub1").path).st_mode),
                 mode)
 
@@ -542,7 +711,7 @@ class FilePathTestCase(AbstractFilePathTestCase):
         self.createLinks()
         self.symlink(self.path.child("file2.link").path,
                      self.path.child("link.link").path)
-        self.assertEquals(self.path.child("link.link").realpath(),
+        self.assertEqual(self.path.child("link.link").realpath(),
                           self.path.child("sub1").child("file2"))
 
 
@@ -562,7 +731,7 @@ class FilePathTestCase(AbstractFilePathTestCase):
         L{FilePath.realpath} returns the path itself if the path is not a
         symbolic link.
         """
-        self.assertEquals(self.path.child("sub1").realpath(),
+        self.assertEqual(self.path.child("sub1").realpath(),
                           self.path.child("sub1"))
 
 
@@ -604,18 +773,18 @@ class FilePathTestCase(AbstractFilePathTestCase):
         def noSymLinks(path):
             return not path.islink()
         x = [foo.path for foo in self.path.walk(descend=noSymLinks)]
-        self.assertEquals(set(x), set(self.all))
+        self.assertEqual(set(x), set(self.all))
 
 
     def test_getAndSet(self):
         content = 'newcontent'
         self.path.child('new').setContent(content)
         newcontent = self.path.child('new').getContent()
-        self.failUnlessEqual(content, newcontent)
+        self.assertEqual(content, newcontent)
         content = 'content'
         self.path.child('new').setContent(content, '.tmp')
         newcontent = self.path.child('new').getContent()
-        self.failUnlessEqual(content, newcontent)
+        self.assertEqual(content, newcontent)
 
 
     def test_getContentFileClosing(self):
@@ -651,7 +820,7 @@ class FilePathTestCase(AbstractFilePathTestCase):
         # Sanity check: setContent should only open one derivative path each
         # time to store the temporary file.
         openedSiblings = fp.openedPaths()
-        self.assertEquals(len(openedSiblings), 2)
+        self.assertEqual(len(openedSiblings), 2)
         self.assertNotEquals(openedSiblings[0], openedSiblings[1])
 
 
@@ -664,13 +833,13 @@ class FilePathTestCase(AbstractFilePathTestCase):
         fp = TrackingFilePath(self.mktemp())
         fp.setContent("hello")
         opened = fp.openedPaths()
-        self.assertEquals(len(opened), 1)
+        self.assertEqual(len(opened), 1)
         self.assertTrue(opened[0].basename().endswith(".new"),
                         "%s does not end with default '.new' extension" % (
                             opened[0].basename()))
         fp.setContent("goodbye", "-something-else")
         opened = fp.openedPaths()
-        self.assertEquals(len(opened), 2)
+        self.assertEqual(len(opened), 2)
         self.assertTrue(opened[1].basename().endswith("-something-else"),
                         "%s does not end with -something-else extension" % (
                             opened[1].basename()))
@@ -705,8 +874,8 @@ class FilePathTestCase(AbstractFilePathTestCase):
         for target, link in targetLinks:
             target.linkTo(link)
             self.assertTrue(link.islink(), "This is a link")
-            self.assertEquals(target.isdir(), link.isdir())
-            self.assertEquals(target.isfile(), link.isfile())
+            self.assertEqual(target.isdir(), link.isdir())
+            self.assertEqual(target.isfile(), link.isfile())
 
 
     def test_linkToErrors(self):
@@ -747,26 +916,26 @@ class FilePathTestCase(AbstractFilePathTestCase):
     def testStatCache(self):
         p = self.path.child('stattest')
         p.touch()
-        self.failUnlessEqual(p.getsize(), 0)
-        self.failUnlessEqual(abs(p.getmtime() - time.time()) // 20, 0)
-        self.failUnlessEqual(abs(p.getctime() - time.time()) // 20, 0)
-        self.failUnlessEqual(abs(p.getatime() - time.time()) // 20, 0)
-        self.failUnlessEqual(p.exists(), True)
-        self.failUnlessEqual(p.exists(), True)
+        self.assertEqual(p.getsize(), 0)
+        self.assertEqual(abs(p.getmtime() - time.time()) // 20, 0)
+        self.assertEqual(abs(p.getctime() - time.time()) // 20, 0)
+        self.assertEqual(abs(p.getatime() - time.time()) // 20, 0)
+        self.assertEqual(p.exists(), True)
+        self.assertEqual(p.exists(), True)
         # OOB removal: FilePath.remove() will automatically restat
         os.remove(p.path)
         # test caching
-        self.failUnlessEqual(p.exists(), True)
+        self.assertEqual(p.exists(), True)
         p.restat(reraise=False)
-        self.failUnlessEqual(p.exists(), False)
-        self.failUnlessEqual(p.islink(), False)
-        self.failUnlessEqual(p.isdir(), False)
-        self.failUnlessEqual(p.isfile(), False)
+        self.assertEqual(p.exists(), False)
+        self.assertEqual(p.islink(), False)
+        self.assertEqual(p.isdir(), False)
+        self.assertEqual(p.isfile(), False)
 
     def testPersist(self):
         newpath = pickle.loads(pickle.dumps(self.path))
-        self.failUnlessEqual(self.path.__class__, newpath.__class__)
-        self.failUnlessEqual(self.path.path, newpath.path)
+        self.assertEqual(self.path.__class__, newpath.__class__)
+        self.assertEqual(self.path.path, newpath.path)
 
     def testInsecureUNIX(self):
         self.assertRaises(filepath.InsecurePath, self.path.child, "..")
@@ -778,10 +947,11 @@ class FilePathTestCase(AbstractFilePathTestCase):
         self.assertRaises(filepath.InsecurePath, self.path.child, r"C:randomfile")
 
     if platform.getType() != 'win32':
-        testInsecureWin32.skip = "Consider yourself lucky."
+        testInsecureWin32.skip = "Test will run only on Windows."
 
     def testInsecureWin32Whacky(self):
-        """Windows has 'special' filenames like NUL and CON and COM1 and LPR
+        """
+        Windows has 'special' filenames like NUL and CON and COM1 and LPR
         and PRN and ... god knows what else.  They can be located anywhere in
         the filesystem.  For obvious reasons, we do not wish to normally permit
         access to these.
@@ -791,10 +961,10 @@ class FilePathTestCase(AbstractFilePathTestCase):
         self.assertRaises(filepath.InsecurePath, self.path.child, r"C:\CON")
 
     if platform.getType() != 'win32':
-        testInsecureWin32Whacky.skip = "Consider yourself lucky."
+        testInsecureWin32Whacky.skip = "Test will run only on Windows."
 
     def testComparison(self):
-        self.assertEquals(filepath.FilePath('a'),
+        self.assertEqual(filepath.FilePath('a'),
                           filepath.FilePath('a'))
         self.failUnless(filepath.FilePath('z') >
                         filepath.FilePath('a'))
@@ -829,14 +999,14 @@ class FilePathTestCase(AbstractFilePathTestCase):
     def testSibling(self):
         p = self.path.child('sibling_start')
         ts = p.sibling('sibling_test')
-        self.assertEquals(ts.dirname(), p.dirname())
-        self.assertEquals(ts.basename(), 'sibling_test')
+        self.assertEqual(ts.dirname(), p.dirname())
+        self.assertEqual(ts.basename(), 'sibling_test')
         ts.createDirectory()
         self.assertIn(ts, self.path.children())
 
     def testTemporarySibling(self):
         ts = self.path.temporarySibling()
-        self.assertEquals(ts.dirname(), self.path.dirname())
+        self.assertEqual(ts.dirname(), self.path.dirname())
         self.assertNotIn(ts.basename(), self.path.listdir())
         ts.createDirectory()
         self.assertIn(ts, self.path.parent().children())
@@ -889,7 +1059,7 @@ class FilePathTestCase(AbstractFilePathTestCase):
         newPaths = list(self.path.walk()) # Record double-copy state
         newPaths.sort()
         oldPaths.sort()
-        self.assertEquals(newPaths, oldPaths)
+        self.assertEqual(newPaths, oldPaths)
 
 
     def test_copyToMissingDestFileClosing(self):
@@ -945,7 +1115,7 @@ class FilePathTestCase(AbstractFilePathTestCase):
         fp = filepath.FilePath(self.mktemp())
         self.path.copyTo(fp)
         self.assertFalse(fp.child("link1").islink())
-        self.assertEquals([x.basename() for x in fp.child("sub1").children()],
+        self.assertEqual([x.basename() for x in fp.child("sub1").children()],
                           [x.basename() for x in fp.child("link1").children()])
 
 
@@ -957,10 +1127,19 @@ class FilePathTestCase(AbstractFilePathTestCase):
         fp = filepath.FilePath(self.mktemp())
         self.path.copyTo(fp, followLinks=False)
         self.assertTrue(fp.child("link1").islink())
-        self.assertEquals(os.readlink(self.path.child("link1").path),
+        self.assertEqual(os.readlink(self.path.child("link1").path),
                           os.readlink(fp.child("link1").path))
 
 
+    def test_copyToMissingSource(self):
+        """
+        If the source path is missing, L{FilePath.copyTo} raises L{OSError}.
+        """
+        path = filepath.FilePath(self.mktemp())
+        exc = self.assertRaises(OSError, path.copyTo, 'some other path')
+        self.assertEqual(exc.errno, errno.ENOENT)
+
+
     def test_moveTo(self):
         """
         Verify that moving an entire directory results into another directory
@@ -973,7 +1152,7 @@ class FilePathTestCase(AbstractFilePathTestCase):
         newPaths = list(self.path.walk()) # Record double-move state
         newPaths.sort()
         oldPaths.sort()
-        self.assertEquals(newPaths, oldPaths)
+        self.assertEqual(newPaths, oldPaths)
 
 
     def test_moveToExistsCache(self):
@@ -989,8 +1168,8 @@ class FilePathTestCase(AbstractFilePathTestCase):
         # Both a sanity check (make sure the file status looks right) and an
         # enticement for stat-caching logic to kick in and remember that these
         # exist / don't exist.
-        self.assertEquals(fp.exists(), True)
-        self.assertEquals(fp2.exists(), False)
+        self.assertEqual(fp.exists(), True)
+        self.assertEqual(fp2.exists(), False)
 
         fp.moveTo(fp2)
         self.assertEqual(fp.exists(), False)
@@ -1103,7 +1282,7 @@ class FilePathTestCase(AbstractFilePathTestCase):
         self.symlink(self.path.child('file1').path, f2.path)
         f2.moveTo(f3)
         self.assertFalse(f3.islink())
-        self.assertEquals(f3.getContent(), 'file 1')
+        self.assertEqual(f3.getContent(), 'file 1')
         self.assertTrue(invokedWith)
 
 
@@ -1118,7 +1297,7 @@ class FilePathTestCase(AbstractFilePathTestCase):
         self.symlink(self.path.child('file1').path, f2.path)
         f2.moveTo(f3, followLinks=False)
         self.assertTrue(f3.islink())
-        self.assertEquals(f3.getContent(), 'file 1')
+        self.assertEqual(f3.getContent(), 'file 1')
         self.assertTrue(invokedWith)
 
 
@@ -1144,7 +1323,7 @@ class FilePathTestCase(AbstractFilePathTestCase):
         # Opening a file for reading when it does not already exist is an error
         nonexistent = self.path.child('nonexistent')
         e = self.assertRaises(IOError, nonexistent.open)
-        self.assertEquals(e.errno, errno.ENOENT)
+        self.assertEqual(e.errno, errno.ENOENT)
 
         # Opening a file for writing when it does not exist is okay
         writer = self.path.child('writer')
@@ -1155,14 +1334,14 @@ class FilePathTestCase(AbstractFilePathTestCase):
         # Make sure those bytes ended up there - and test opening a file for
         # reading when it does exist at the same time
         f = writer.open()
-        self.assertEquals(f.read(), 'abc\ndef')
+        self.assertEqual(f.read(), 'abc\ndef')
         f.close()
 
         # Re-opening that file in write mode should erase whatever was there.
         f = writer.open('w')
         f.close()
         f = writer.open()
-        self.assertEquals(f.read(), '')
+        self.assertEqual(f.read(), '')
         f.close()
 
         # Put some bytes in a file so we can test that appending does not
@@ -1177,12 +1356,12 @@ class FilePathTestCase(AbstractFilePathTestCase):
         f.close()
 
         f = appender.open('r')
-        self.assertEquals(f.read(), 'abcdef')
+        self.assertEqual(f.read(), 'abcdef')
         f.close()
 
         # read/write should let us do both without erasing those bytes
         f = appender.open('r+')
-        self.assertEquals(f.read(), 'abcdef')
+        self.assertEqual(f.read(), 'abcdef')
         # ANSI C *requires* an fseek or an fgetpos between an fread and an
         # fwrite or an fwrite and a fread.  We can't reliable get Python to
         # invoke fgetpos, so we seek to a 0 byte offset from the current
@@ -1196,13 +1375,13 @@ class FilePathTestCase(AbstractFilePathTestCase):
 
         # Make sure those new bytes really showed up
         f = appender.open('r')
-        self.assertEquals(f.read(), 'abcdefghi')
+        self.assertEqual(f.read(), 'abcdefghi')
         f.close()
 
         # write/read should let us do both, but erase anything that's there
         # already.
         f = appender.open('w+')
-        self.assertEquals(f.read(), '')
+        self.assertEqual(f.read(), '')
         f.seek(0, 1) # Don't forget this!
         f.write('123')
         f.close()
@@ -1215,10 +1394,10 @@ class FilePathTestCase(AbstractFilePathTestCase):
         # The cursor is not at the end of the file until after the first write.
         f.write('456')
         f.seek(0, 1) # Asinine.
-        self.assertEquals(f.read(), '')
+        self.assertEqual(f.read(), '')
 
         f.seek(0, 0)
-        self.assertEquals(f.read(), '123456')
+        self.assertEqual(f.read(), '123456')
         f.close()
 
         # Opening a file exclusively must fail if that file exists already.
@@ -1268,10 +1447,10 @@ class FilePathTestCase(AbstractFilePathTestCase):
         an operation has occurred in the mean time.
         """
         fp = filepath.FilePath(self.mktemp())
-        self.assertEquals(fp.exists(), False)
+        self.assertEqual(fp.exists(), False)
 
         fp.makedirs()
-        self.assertEquals(fp.exists(), True)
+        self.assertEqual(fp.exists(), True)
 
 
     def test_changed(self):
@@ -1282,7 +1461,7 @@ class FilePathTestCase(AbstractFilePathTestCase):
         """
         fp = filepath.FilePath(self.mktemp())
         fp.setContent("12345")
-        self.assertEquals(fp.getsize(), 5)
+        self.assertEqual(fp.getsize(), 5)
 
         # Someone else comes along and changes the file.
         fObj = open(fp.path, 'wb')
@@ -1290,13 +1469,119 @@ class FilePathTestCase(AbstractFilePathTestCase):
         fObj.close()
 
         # Sanity check for caching: size should still be 5.
-        self.assertEquals(fp.getsize(), 5)
+        self.assertEqual(fp.getsize(), 5)
         fp.changed()
 
         # This path should look like we don't know what status it's in, not that
         # we know that it didn't exist when last we checked.
         self.assertEqual(fp.statinfo, None)
-        self.assertEquals(fp.getsize(), 8)
+        self.assertEqual(fp.getsize(), 8)
+
+
+    def test_getPermissions_POSIX(self):
+        """
+        Getting permissions for a file returns a L{Permissions} object for
+        POSIX platforms (which supports separate user, group, and other
+        permissions bits.
+        """
+        for mode in (0777, 0700):
+            self.path.child("sub1").chmod(mode)
+            self.assertEqual(self.path.child("sub1").getPermissions(),
+                              filepath.Permissions(mode))
+        self.path.child("sub1").chmod(0764) #sanity check
+        self.assertEqual(self.path.child("sub1").getPermissions().shorthand(),
+                          "rwxrw-r--")
+
+
+    def test_getPermissions_Windows(self):
+        """
+        Getting permissions for a file returns a L{Permissions} object in
+        Windows.  Windows requires a different test, because user permissions
+        = group permissions = other permissions.  Also, chmod may not be able
+        to set the execute bit, so we are skipping tests that set the execute
+        bit.
+        """
+        for mode in (0777, 0555):
+            self.path.child("sub1").chmod(mode)
+            self.assertEqual(self.path.child("sub1").getPermissions(),
+                              filepath.Permissions(mode))
+        self.path.child("sub1").chmod(0511) #sanity check to make sure that
+        # user=group=other permissions
+        self.assertEqual(self.path.child("sub1").getPermissions().shorthand(),
+                          "r-xr-xr-x")
+
+
+    def test_whetherBlockOrSocket(self):
+        """
+        Ensure that a file is not a block or socket
+        """
+        self.assertFalse(self.path.isBlockDevice())
+        self.assertFalse(self.path.isSocket())
+
+
+    def test_statinfoBitsNotImplementedInWindows(self):
+        """
+        Verify that certain file stats are not available on Windows
+        """
+        self.assertRaises(NotImplementedError, self.path.getInodeNumber)
+        self.assertRaises(NotImplementedError, self.path.getDevice)
+        self.assertRaises(NotImplementedError, self.path.getNumberOfHardLinks)
+        self.assertRaises(NotImplementedError, self.path.getUserID)
+        self.assertRaises(NotImplementedError, self.path.getGroupID)
+
+
+    def test_statinfoBitsAreNumbers(self):
+        """
+        Verify that file inode/device/nlinks/uid/gid stats are numbers in
+        a POSIX environment
+        """
+        c = self.path.child('file1')
+        for p in self.path, c:
+            self.assertIsInstance(p.getInodeNumber(), long)
+            self.assertIsInstance(p.getDevice(), long)
+            self.assertIsInstance(p.getNumberOfHardLinks(), int)
+            self.assertIsInstance(p.getUserID(), int)
+            self.assertIsInstance(p.getGroupID(), int)
+        self.assertEqual(self.path.getUserID(), c.getUserID())
+        self.assertEqual(self.path.getGroupID(), c.getGroupID())
+
+
+    def test_statinfoNumbersAreValid(self):
+        """
+        Verify that the right numbers come back from the right accessor methods
+        for file inode/device/nlinks/uid/gid (in a POSIX environment)
+        """
+        # specify fake statinfo information
+        class FakeStat:
+            st_ino = 200
+            st_dev = 300
+            st_nlink = 400
+            st_uid = 500
+            st_gid = 600
+
+        # monkey patch in a fake restat method for self.path
+        fake = FakeStat()
+        def fakeRestat(*args, **kwargs):
+            self.path.statinfo = fake
+        self.path.restat = fakeRestat
+
+        # ensure that restat will need to be called to get values
+        self.path.statinfo = None
+
+        self.assertEqual(self.path.getInodeNumber(), fake.st_ino)
+        self.assertEqual(self.path.getDevice(), fake.st_dev)
+        self.assertEqual(self.path.getNumberOfHardLinks(), fake.st_nlink)
+        self.assertEqual(self.path.getUserID(), fake.st_uid)
+        self.assertEqual(self.path.getGroupID(), fake.st_gid)
+
+
+    if platform.isWindows():
+        test_statinfoBitsAreNumbers.skip = True
+        test_statinfoNumbersAreValid.skip = True
+        test_getPermissions_POSIX.skip = True
+    else:
+        test_statinfoBitsNotImplementedInWindows.skip = "Test will run only on Windows."
+        test_getPermissions_Windows.skip = "Test will run only on Windows."
 
 
 
@@ -1307,31 +1592,31 @@ class URLPathTestCase(unittest.TestCase):
         self.path = urlpath.URLPath.fromString("http://example.com/foo/bar?yes=no&no=yes#footer")
 
     def testStringConversion(self):
-        self.assertEquals(str(self.path), "http://example.com/foo/bar?yes=no&no=yes#footer")
+        self.assertEqual(str(self.path), "http://example.com/foo/bar?yes=no&no=yes#footer")
 
     def testChildString(self):
-        self.assertEquals(str(self.path.child('hello')), "http://example.com/foo/bar/hello")
-        self.assertEquals(str(self.path.child('hello').child('')), "http://example.com/foo/bar/hello/")
+        self.assertEqual(str(self.path.child('hello')), "http://example.com/foo/bar/hello")
+        self.assertEqual(str(self.path.child('hello').child('')), "http://example.com/foo/bar/hello/")
 
     def testSiblingString(self):
-        self.assertEquals(str(self.path.sibling('baz')), 'http://example.com/foo/baz')
+        self.assertEqual(str(self.path.sibling('baz')), 'http://example.com/foo/baz')
 
         # The sibling of http://example.com/foo/bar/
         #     is http://example.comf/foo/bar/baz
         # because really we are constructing a sibling of
         # http://example.com/foo/bar/index.html
-        self.assertEquals(str(self.path.child('').sibling('baz')), 'http://example.com/foo/bar/baz')
+        self.assertEqual(str(self.path.child('').sibling('baz')), 'http://example.com/foo/bar/baz')
 
     def testParentString(self):
         # parent should be equivalent to '..'
         # 'foo' is the current directory, '/' is the parent directory
-        self.assertEquals(str(self.path.parent()), 'http://example.com/')
-        self.assertEquals(str(self.path.child('').parent()), 'http://example.com/foo/')
-        self.assertEquals(str(self.path.child('baz').parent()), 'http://example.com/foo/')
-        self.assertEquals(str(self.path.parent().parent().parent().parent().parent()), 'http://example.com/')
+        self.assertEqual(str(self.path.parent()), 'http://example.com/')
+        self.assertEqual(str(self.path.child('').parent()), 'http://example.com/foo/')
+        self.assertEqual(str(self.path.child('baz').parent()), 'http://example.com/foo/')
+        self.assertEqual(str(self.path.parent().parent().parent().parent().parent()), 'http://example.com/')
 
     def testHereString(self):
         # here should be equivalent to '.'
-        self.assertEquals(str(self.path.here()), 'http://example.com/foo/')
-        self.assertEquals(str(self.path.child('').here()), 'http://example.com/foo/bar/')
+        self.assertEqual(str(self.path.here()), 'http://example.com/foo/')
+        self.assertEqual(str(self.path.child('').here()), 'http://example.com/foo/bar/')
 
