@@ -6,7 +6,10 @@ Test cases for twisted.reflect module.
 """
 
 import weakref, os
-from ihooks import ModuleImporter
+try:
+    from ihooks import ModuleImporter
+except ImportError:
+    ModuleImporter = None
 
 try:
     from collections import deque
@@ -294,6 +297,9 @@ class ImportHooksLookupTests(LookupsTestCase):
     Runs all of the tests from L{LookupsTestCase} after installing a custom
     import hook.
     """
+    if ModuleImporter == None:
+        skip = "ihooks not available"
+
     def setUp(self):
         """
         Perturb the normal import behavior subtly by installing an import
@@ -757,27 +763,111 @@ class DeprecationTestCase(unittest.TestCase):
     Test deprecations in twisted.python.reflect
     """
 
-    def test_macro(self):
-        """
-        Test deprecation of L{reflect.macro}.
-        """
-        result = self.callDeprecated(Version("Twisted", 8, 2, 0),
-            reflect.macro, "test", __file__, "test = 1")
-        self.assertEqual(result, 1)
-
     def test_allYourBase(self):
         """
-        Test deprecation of L{reflect.allYourBase}.
+        Test deprecation of L{reflect.allYourBase}. See #5481 for removal.
         """
         self.callDeprecated(
             (Version("Twisted", 11, 0, 0), "inspect.getmro"),
             reflect.allYourBase, DeprecationTestCase)
 
+
     def test_accumulateBases(self):
         """
-        Test deprecation of L{reflect.accumulateBases}.
+        Test deprecation of L{reflect.accumulateBases}. See #5481 for removal.
         """
         l = []
         self.callDeprecated(
             (Version("Twisted", 11, 0, 0), "inspect.getmro"),
             reflect.accumulateBases, DeprecationTestCase, l, None)
+
+
+    def lookForDeprecationWarning(self, testMethod, attributeName, warningMsg):
+        """
+        Test deprecation of attribute 'reflect.attributeName' by calling
+        'reflect.testMethod' and verifying the warning message
+        'reflect.warningMsg'
+
+        @param testMethod: Name of the offending function to be used with
+            flushWarnings
+        @type testmethod: C{str}
+
+        @param attributeName: Name of attribute to be checked for deprecation
+        @type attributeName: C{str}
+
+        @param warningMsg: Deprecation warning message
+        @type warningMsg: C{str}
+        """
+        warningsShown = self.flushWarnings([testMethod])
+        self.assertEqual(len(warningsShown), 1)
+        self.assertIdentical(warningsShown[0]['category'], DeprecationWarning)
+        self.assertEqual(
+            warningsShown[0]['message'],
+            "twisted.python.reflect." + attributeName + " "
+            "was deprecated in Twisted 12.1.0: " + warningMsg + ".")
+
+
+    def test_settable(self):
+        """
+        Test deprecation of L{reflect.Settable}.
+        """
+        reflect.Settable()
+        self.lookForDeprecationWarning(
+            self.test_settable, "Settable",
+            "Settable is old and untested. Please write your own version of this "
+            "functionality if you need it")
+
+
+    def test_accessorType(self):
+        """
+        Test deprecation of L{reflect.AccessorType}.
+        """
+        reflect.AccessorType(' ', ( ), { })
+        self.lookForDeprecationWarning(
+            self.test_accessorType, "AccessorType",
+            "AccessorType is old and untested. Please write your own version of "
+            "this functionality if you need it")
+
+
+    def test_propertyAccessor(self):
+        """
+        Test deprecation of L{reflect.PropertyAccessor}.
+        """
+        reflect.PropertyAccessor()
+        self.lookForDeprecationWarning(
+            self.test_propertyAccessor, "PropertyAccessor",
+            "PropertyAccessor is old and untested. Please write your own "
+            "version of this functionality if you need it")
+
+
+    def test_accessor(self):
+        """
+        Test deprecation of L{reflect.Accessor}.
+        """
+        reflect.Accessor()
+        self.lookForDeprecationWarning(
+            self.test_accessor, "Accessor",
+            "Accessor is an implementation for Python 2.1 which is no longer "
+            "supported by Twisted")
+
+
+    def test_originalAccessor(self):
+        """
+        Test deprecation of L{reflect.OriginalAccessor}.
+        """
+        reflect.OriginalAccessor()
+        self.lookForDeprecationWarning(
+            self.test_originalAccessor, "OriginalAccessor",
+            "OriginalAccessor is a reference to class "
+            "twisted.python.reflect.Accessor which is deprecated")
+
+
+    def test_summer(self):
+        """
+        Test deprecation of L{reflect.Summer}.
+        """
+        reflect.Summer()
+        self.lookForDeprecationWarning(
+            self.test_summer, "Summer",
+            "Summer is a child class of twisted.python.reflect.Accessor which "
+            "is deprecated")
