@@ -1,4 +1,4 @@
-# Copyright (c) 2001-2008 Twisted Matrix Laboratories.
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
@@ -235,7 +235,7 @@ class ListeningTestCase(unittest.TestCase):
         self.addCleanup(connector.disconnect)
         def check((serverProto, clientProto)):
             portNumber = port.getHost().port
-            self.assertEquals(
+            self.assertEqual(
                 repr(serverProto.transport),
                 "<AccumulatingProtocol #0 on %s>" % (portNumber,))
             serverProto.transport.loseConnection()
@@ -434,11 +434,11 @@ class LoopbackTestCase(unittest.TestCase):
         def check(serverProtocol, clientProtocol):
             for p in [serverProtocol, clientProtocol]:
                 transport = p.transport
-                self.assertEquals(transport.getTcpNoDelay(), 0)
+                self.assertEqual(transport.getTcpNoDelay(), 0)
                 transport.setTcpNoDelay(1)
-                self.assertEquals(transport.getTcpNoDelay(), 1)
+                self.assertEqual(transport.getTcpNoDelay(), 1)
                 transport.setTcpNoDelay(0)
-                self.assertEquals(transport.getTcpNoDelay(), 0)
+                self.assertEqual(transport.getTcpNoDelay(), 0)
         return self._connectedClientAndServerTest(check)
 
 
@@ -452,11 +452,11 @@ class LoopbackTestCase(unittest.TestCase):
         def check(serverProtocol, clientProtocol):
             for p in [serverProtocol, clientProtocol]:
                 transport = p.transport
-                self.assertEquals(transport.getTcpKeepAlive(), 0)
+                self.assertEqual(transport.getTcpKeepAlive(), 0)
                 transport.setTcpKeepAlive(1)
-                self.assertEquals(transport.getTcpKeepAlive(), 1)
+                self.assertEqual(transport.getTcpKeepAlive(), 1)
                 transport.setTcpKeepAlive(0)
-                self.assertEquals(transport.getTcpKeepAlive(), 0)
+                self.assertEqual(transport.getTcpKeepAlive(), 0)
         return self._connectedClientAndServerTest(check)
 
 
@@ -602,6 +602,10 @@ class ClientStartStopFactory(MyClientFactory):
     started = 0
     stopped = 0
 
+    def __init__(self, *a, **kw):
+        MyClientFactory.__init__(self, *a, **kw)
+        self.whenStopped = defer.Deferred()
+
     def startFactory(self):
         if self.started or self.stopped:
             raise RuntimeError
@@ -611,6 +615,7 @@ class ClientStartStopFactory(MyClientFactory):
         if not self.started or self.stopped:
             raise RuntimeError
         self.stopped = 1
+        self.whenStopped.callback(True)
 
 
 class FactoryTestCase(unittest.TestCase):
@@ -654,7 +659,7 @@ class FactoryTestCase(unittest.TestCase):
         closedDeferred.addCallback(cbClosed)
 
         def cbClosedAll(ignored):
-            self.assertEquals((f.started, f.stopped), (1, 1))
+            self.assertEqual((f.started, f.stopped), (1, 1))
         closedDeferred.addCallback(cbClosedAll)
         return closedDeferred
 
@@ -677,99 +682,6 @@ class FactoryTestCase(unittest.TestCase):
 
 
 
-class ConnectorTestCase(unittest.TestCase):
-
-    def test_connectorIdentity(self):
-        """
-        L{IReactorTCP.connectTCP} returns an object which provides
-        L{IConnector}.  The destination of the connector is the address which
-        was passed to C{connectTCP}.  The same connector object is passed to
-        the factory's C{startedConnecting} method as to the factory's
-        C{clientConnectionLost} method.
-        """
-        serverFactory = ClosingFactory()
-        tcpPort = reactor.listenTCP(0, serverFactory, interface="127.0.0.1")
-        serverFactory.port = tcpPort
-        self.addCleanup(serverFactory.cleanUp)
-        portNumber = tcpPort.getHost().port
-
-        seenConnectors = []
-        seenFailures = []
-
-        clientFactory = ClientStartStopFactory()
-        clientFactory.clientConnectionLost = (
-            lambda connector, reason: (seenConnectors.append(connector),
-                                       seenFailures.append(reason)))
-        clientFactory.startedConnecting = seenConnectors.append
-
-        connector = reactor.connectTCP("127.0.0.1", portNumber, clientFactory)
-        self.assertTrue(interfaces.IConnector.providedBy(connector))
-        dest = connector.getDestination()
-        self.assertEquals(dest.type, "TCP")
-        self.assertEquals(dest.host, "127.0.0.1")
-        self.assertEquals(dest.port, portNumber)
-
-        d = loopUntil(lambda: clientFactory.stopped)
-        def clientFactoryStopped(ignored):
-            seenFailures[0].trap(error.ConnectionDone)
-            self.assertEqual(seenConnectors, [connector, connector])
-        d.addCallback(clientFactoryStopped)
-        return d
-
-
-    def test_userFail(self):
-        """
-        Calling L{IConnector.stopConnecting} in C{Factory.startedConnecting}
-        results in C{Factory.clientConnectionFailed} being called with
-        L{error.UserError} as the reason.
-        """
-        serverFactory = MyServerFactory()
-        tcpPort = reactor.listenTCP(0, serverFactory, interface="127.0.0.1")
-        self.addCleanup(tcpPort.stopListening)
-        portNumber = tcpPort.getHost().port
-
-        def startedConnecting(connector):
-            connector.stopConnecting()
-
-        clientFactory = ClientStartStopFactory()
-        clientFactory.startedConnecting = startedConnecting
-        reactor.connectTCP("127.0.0.1", portNumber, clientFactory)
-
-        d = loopUntil(lambda: clientFactory.stopped)
-        def check(ignored):
-            self.assertEquals(clientFactory.failed, 1)
-            clientFactory.reason.trap(error.UserError)
-        return d.addCallback(check)
-
-
-    def test_reconnect(self):
-        """
-        Calling L{IConnector.connect} in C{Factory.clientConnectionLost} causes
-        a new connection attempt to be made.
-        """
-        serverFactory = ClosingFactory()
-        tcpPort = reactor.listenTCP(0, serverFactory, interface="127.0.0.1")
-        serverFactory.port = tcpPort
-        self.addCleanup(serverFactory.cleanUp)
-        portNumber = tcpPort.getHost().port
-
-        clientFactory = MyClientFactory()
-
-        def clientConnectionLost(connector, reason):
-            connector.connect()
-        clientFactory.clientConnectionLost = clientConnectionLost
-        reactor.connectTCP("127.0.0.1", portNumber, clientFactory)
-
-        d = loopUntil(lambda: clientFactory.failed)
-        def reconnectFailed(ignored):
-            p = clientFactory.protocol
-            self.assertEqual((p.made, p.closed), (1, 1))
-            clientFactory.reason.trap(error.ConnectionRefusedError)
-            self.assertEqual(clientFactory.stopped, 1)
-        return d.addCallback(reconnectFailed)
-
-
-
 class CannotBindTestCase(unittest.TestCase):
     """
     Tests for correct behavior when a reactor cannot bind to the required TCP
@@ -787,9 +699,9 @@ class CannotBindTestCase(unittest.TestCase):
         self.addCleanup(p1.stopListening)
         n = p1.getHost().port
         dest = p1.getHost()
-        self.assertEquals(dest.type, "TCP")
-        self.assertEquals(dest.host, "127.0.0.1")
-        self.assertEquals(dest.port, n)
+        self.assertEqual(dest.type, "TCP")
+        self.assertEqual(dest.host, "127.0.0.1")
+        self.assertEqual(dest.port, n)
 
         # make sure new listen raises error
         self.assertRaises(error.CannotListenError,
@@ -837,7 +749,7 @@ class CannotBindTestCase(unittest.TestCase):
             return d
 
         def _check1connect2(results, cf1):
-            self.assertEquals(cf1.protocol.made, 1)
+            self.assertEqual(cf1.protocol.made, 1)
 
             d1 = defer.Deferred()
             d2 = defer.Deferred()
@@ -855,13 +767,13 @@ class CannotBindTestCase(unittest.TestCase):
             return dl
 
         def _check2failed(results, cf1, cf2):
-            self.assertEquals(cf2.failed, 1)
+            self.assertEqual(cf2.failed, 1)
             cf2.reason.trap(error.ConnectBindError)
             self.assertTrue(cf2.reason.check(error.ConnectBindError))
             return results
 
         def _check2stopped(results, cf1, cf2):
-            self.assertEquals(cf2.stopped, 1)
+            self.assertEqual(cf2.stopped, 1)
             return results
 
         def _stop(results, cf1, cf2):
@@ -872,7 +784,7 @@ class CannotBindTestCase(unittest.TestCase):
             return d
 
         def _check1cleanup(results, cf1):
-            self.assertEquals(cf1.stopped, 1)
+            self.assertEqual(cf1.stopped, 1)
 
         theDeferred.addCallback(_connect1)
         return theDeferred
@@ -909,8 +821,8 @@ class LocalRemoteAddressTestCase(unittest.TestCase):
         connector = reactor.connectTCP('127.0.0.1', n, clientFactory)
 
         def check(ignored):
-            self.assertEquals([port.getHost()], clientFactory.peerAddresses)
-            self.assertEquals(
+            self.assertEqual([port.getHost()], clientFactory.peerAddresses)
+            self.assertEqual(
                 port.getHost(), clientFactory.protocol.transport.getPeer())
         onConnection.addCallback(check)
 
@@ -1447,25 +1359,6 @@ class AddressTestCase(unittest.TestCase):
             [IPv4Address('TCP', clientPeer.host, clientPeer.port)])
 
 
-    def test_buildProtocolServer(self):
-        """
-        L{ServerFactory.buildProtocol} should be invoked with the address of
-        the client which has connected to the port the factory is listening on,
-        which should be the same as the address reported by the C{getPeer}
-        method of the transport of the server protocol and as the C{getHost}
-        method of the transport of the client protocol.
-        """
-        clientHost = self.client.protocol.transport.getHost()
-        serverPeer = self.server.protocol.transport.getPeer()
-
-        self.assertEqual(
-            self.serverWrapper.addresses,
-            [IPv4Address('TCP', serverPeer.host, serverPeer.port)])
-        self.assertEqual(
-            self.serverWrapper.addresses,
-            [IPv4Address('TCP', clientHost.host, clientHost.port)])
-
-
 
 class LargeBufferWriterProtocol(protocol.Protocol):
 
@@ -1594,12 +1487,12 @@ class HalfCloseTestCase(unittest.TestCase):
     def _setUp(self, client):
         self.client = client
         self.clientProtoConnectionLost = self.client.closedDeferred = defer.Deferred()
-        self.assertEquals(self.client.transport.connected, 1)
+        self.assertEqual(self.client.transport.connected, 1)
         # Wait for the server to notice there is a connection, too.
         return loopUntil(lambda: getattr(self.f, 'protocol', None) is not None)
 
     def tearDown(self):
-        self.assertEquals(self.client.closed, 0)
+        self.assertEqual(self.client.closed, 0)
         self.client.transport.loseConnection()
         d = defer.maybeDeferred(self.p.stopListening)
         d.addCallback(lambda ign: self.clientProtoConnectionLost)
@@ -1607,17 +1500,17 @@ class HalfCloseTestCase(unittest.TestCase):
         return d
 
     def _tearDown(self, ignored):
-        self.assertEquals(self.client.closed, 1)
+        self.assertEqual(self.client.closed, 1)
         # because we did half-close, the server also needs to
         # closed explicitly.
-        self.assertEquals(self.f.protocol.closed, 0)
+        self.assertEqual(self.f.protocol.closed, 0)
         d = defer.Deferred()
         def _connectionLost(reason):
             self.f.protocol.closed = 1
             d.callback(None)
         self.f.protocol.connectionLost = _connectionLost
         self.f.protocol.transport.loseConnection()
-        d.addCallback(lambda x:self.assertEquals(self.f.protocol.closed, 1))
+        d.addCallback(lambda x:self.assertEqual(self.f.protocol.closed, 1))
         return d
 
     def testCloseWriteCloser(self):
@@ -1631,18 +1524,18 @@ class HalfCloseTestCase(unittest.TestCase):
             t.loseWriteConnection()
             return loopUntil(lambda :t._writeDisconnected)
         def check(ignored):
-            self.assertEquals(client.closed, False)
-            self.assertEquals(client.writeHalfClosed, True)
-            self.assertEquals(client.readHalfClosed, False)
+            self.assertEqual(client.closed, False)
+            self.assertEqual(client.writeHalfClosed, True)
+            self.assertEqual(client.readHalfClosed, False)
             return loopUntil(lambda :f.protocol.readHalfClosed)
         def write(ignored):
             w = client.transport.write
             w(" world")
             w("lalala fooled you")
-            self.assertEquals(0, len(client.transport._tempDataBuffer))
-            self.assertEquals(f.protocol.data, "hello")
-            self.assertEquals(f.protocol.closed, False)
-            self.assertEquals(f.protocol.readHalfClosed, True)
+            self.assertEqual(0, len(client.transport._tempDataBuffer))
+            self.assertEqual(f.protocol.data, "hello")
+            self.assertEqual(f.protocol.closed, False)
+            self.assertEqual(f.protocol.readHalfClosed, True)
         return d.addCallback(loseWrite).addCallback(check).addCallback(write)
 
     def testWriteCloseNotification(self):
@@ -1652,7 +1545,7 @@ class HalfCloseTestCase(unittest.TestCase):
         d = defer.gatherResults([
             loopUntil(lambda :f.protocol.writeHalfClosed),
             loopUntil(lambda :self.client.readHalfClosed)])
-        d.addCallback(lambda _: self.assertEquals(
+        d.addCallback(lambda _: self.assertEqual(
             f.protocol.readHalfClosed, False))
         return d
 
@@ -1708,7 +1601,7 @@ class HalfClose2TestCase(unittest.TestCase):
         self.f.protocol.closedDeferred = d = defer.Deferred()
         self.client.closedDeferred = d2 = defer.Deferred()
         d.addCallback(lambda x:
-                      self.failUnlessEqual(self.f.protocol.closed, True))
+                      self.assertEqual(self.f.protocol.closed, True))
         return defer.gatherResults([d, d2])
 
 
@@ -1804,11 +1697,11 @@ class LogTestCase(unittest.TestCase):
         self.addCleanup(connector.disconnect)
 
         # It should still have the default value
-        self.assertEquals(connector.transport.logstr,
+        self.assertEqual(connector.transport.logstr,
                           "Uninitialized")
 
         def cb(ign):
-            self.assertEquals(connector.transport.logstr,
+            self.assertEqual(connector.transport.logstr,
                               "AccumulatingProtocol,client")
         client.protocolConnectionMade.addCallback(cb)
         return client.protocolConnectionMade
@@ -1909,7 +1802,7 @@ class CallBackOrderTestCase(unittest.TestCase):
             return 'CL'
 
         def _cbGather(res):
-            self.assertEquals(res, ['CL', 'CCL'])
+            self.assertEqual(res, ['CL', 'CCL'])
 
         d = defer.gatherResults([
                 client.protocolConnectionLost.addCallback(_cbCL),
