@@ -1,12 +1,22 @@
-# Copyright (c) 2001-2011 Twisted Matrix Laboratories.
-# See LICENSE for details
+# Copyright (c) Twisted Matrix Laboratories.
+# See LICENSE for details.
 
 """
-Tests for assertions provided by L{twisted.trial.unittest.TestCase}.
+Tests for assertions provided by C{SynchronousTestCase} and C{TestCase},
+provided by L{twisted.trial.unittest}.
+
+L{TestFailureTests} demonstrates that L{SynchronousTestCase.fail} works, so that
+is the only method on C{twisted.trial.unittest.SynchronousTestCase} that is
+initially assumed to work.  The test classes are arranged so that the methods
+demonstrated to work earlier in the file are used by those later in the file
+(even though the runner will probably not run the tests in this order).
 """
 
+from __future__ import division
+
 import warnings
 from pprint import pformat
+import unittest as pyunit
 
 from twisted.python import reflect, failure
 from twisted.python.deprecate import deprecated, getVersionString
@@ -27,83 +37,249 @@ class MockEquality(object):
         return self.name[0] == other.name[0]
 
 
-class TestAssertions(unittest.TestCase):
+class TestFailureTests(pyunit.TestCase):
     """
-    Tests for TestCase's assertion methods.  That is, failUnless*,
-    failIf*, assert*.
+    Tests for the most basic functionality of L{SynchronousTestCase}, for
+    failing tests.
+
+    This class contains tests to demonstrate that L{SynchronousTestCase.fail}
+    can be used to fail a test, and that that failure is reflected in the test
+    result object.  This should be sufficient functionality so that further
+    tests can be built on L{SynchronousTestCase} instead of
+    L{unittest.TestCase}.  This depends on L{unittest.TestCase} working.
+    """
+    class FailingTest(unittest.SynchronousTestCase):
+        def test_fails(self):
+            self.fail("This test fails.")
+
+
+    def setUp(self):
+        """
+        Load a suite of one test which can be used to exercise the failure
+        handling behavior.
+        """
+        components = [
+            __name__, self.__class__.__name__, self.FailingTest.__name__]
+        self.loader = pyunit.TestLoader()
+        self.suite = self.loader.loadTestsFromName(".".join(components))
+        self.test = list(self.suite)[0]
+
+
+    def test_fail(self):
+        """
+        L{SynchronousTestCase.fail} raises
+        L{SynchronousTestCase.failureException} with the given argument.
+        """
+        try:
+            self.test.fail("failed")
+        except self.test.failureException as result:
+            self.assertEqual("failed", str(result))
+        else:
+            self.fail(
+                "SynchronousTestCase.fail method did not raise "
+                "SynchronousTestCase.failureException")
+
+
+    def test_failingExceptionFails(self):
+        """
+        When a test method raises L{SynchronousTestCase.failureException}, the test is
+        marked as having failed on the L{TestResult}.
+        """
+        result = pyunit.TestResult()
+        self.suite.run(result)
+        self.failIf(result.wasSuccessful())
+        self.assertEqual(result.errors, [])
+        self.assertEqual(len(result.failures), 1)
+        self.assertEqual(result.failures[0][0], self.test)
+
+
+
+class AssertFalseTests(unittest.SynchronousTestCase):
+    """
+    Tests for L{SynchronousTestCase}'s C{assertFalse} and C{failIf} assertion
+    methods.
 
     This is pretty paranoid.  Still, a certain paranoia is healthy if you
     are testing a unit testing framework.
+
+    @note: As of 11.2, C{assertFalse} is preferred over C{failIf}.
     """
+    def _assertFalseFalse(self, method):
+        """
+        Perform the positive case test for C{failIf} or C{assertFalse}.
 
-    class FailingTest(unittest.TestCase):
-        def test_fails(self):
-            raise self.failureException()
-
-    def testFail(self):
-        try:
-            self.fail("failed")
-        except self.failureException, e:
-            if not str(e) == 'failed':
-                raise self.failureException("Exception had msg %s instead of %s"
-                                            % str(e), 'failed')
-        else:
-            raise self.failureException("Call to self.fail() didn't fail test")
-
-    def test_failingException_fails(self):
-        test = runner.TestLoader().loadClass(TestAssertions.FailingTest)
-        result = reporter.TestResult()
-        test.run(result)
-        self.failIf(result.wasSuccessful())
-        self.failUnlessEqual(result.errors, [])
-        self.failUnlessEqual(len(result.failures), 1)
-
-    def test_failIf(self):
+        @param method: The test method to test.
+        """
         for notTrue in [0, 0.0, False, None, (), []]:
-            self.failIf(notTrue, "failed on %r" % (notTrue,))
+            result = method(notTrue, "failed on %r" % (notTrue,))
+            if result != notTrue:
+                self.fail("Did not return argument %r" % (notTrue,))
+
+
+    def _assertFalseTrue(self, method):
+        """
+        Perform the negative case test for C{failIf} or C{assertFalse}.
+
+        @param method: The test method to test.
+        """
         for true in [1, True, 'cat', [1,2], (3,4)]:
             try:
-                self.failIf(true, "failed on %r" % (true,))
+                method(true, "failed on %r" % (true,))
             except self.failureException, e:
-                self.failUnlessEqual(str(e), "failed on %r" % (true,))
+                if str(e) != "failed on %r" % (true,):
+                    self.fail("Raised incorrect exception on %r: %r" % (true, e))
             else:
                 self.fail("Call to failIf(%r) didn't fail" % (true,))
 
-    def test_failUnless(self):
+
+    def test_failIfFalse(self):
+        """
+        L{SynchronousTestCase.failIf} returns its argument if its argument is
+        not considered true.
+        """
+        self._assertFalseFalse(self.failIf)
+
+
+    def test_assertFalseFalse(self):
+        """
+        L{SynchronousTestCase.assertFalse} returns its argument if its argument
+        is not considered true.
+        """
+        self._assertFalseFalse(self.assertFalse)
+
+
+    def test_failIfTrue(self):
+        """
+        L{SynchronousTestCase.failIf} raises
+        L{SynchronousTestCase.failureException} if its argument is considered
+        true.
+        """
+        self._assertFalseTrue(self.failIf)
+
+
+    def test_assertFalseTrue(self):
+        """
+        L{SynchronousTestCase.assertFalse} raises
+        L{SynchronousTestCase.failureException} if its argument is considered
+        true.
+        """
+        self._assertFalseTrue(self.assertFalse)
+
+
+
+class AssertTrueTests(unittest.SynchronousTestCase):
+    """
+    Tests for L{SynchronousTestCase}'s C{assertTrue} and C{failUnless} assertion
+    methods.
+
+    This is pretty paranoid.  Still, a certain paranoia is healthy if you
+    are testing a unit testing framework.
+
+    @note: As of 11.2, C{assertTrue} is preferred over C{failUnless}.
+    """
+    def _assertTrueFalse(self, method):
+        """
+        Perform the negative case test for C{assertTrue} and C{failUnless}.
+
+        @param method: The test method to test.
+        """
         for notTrue in [0, 0.0, False, None, (), []]:
             try:
-                self.failUnless(notTrue, "failed on %r" % (notTrue,))
+                method(notTrue, "failed on %r" % (notTrue,))
             except self.failureException, e:
-                self.failUnlessEqual(str(e), "failed on %r" % (notTrue,))
+                if str(e) != "failed on %r" % (notTrue,):
+                    self.fail(
+                        "Raised incorrect exception on %r: %r" % (notTrue, e))
             else:
-                self.fail("Call to failUnless(%r) didn't fail" % (notTrue,))
-        for true in [1, True, 'cat', [1,2], (3,4)]:
-            self.failUnless(true, "failed on %r" % (true,))
+                self.fail(
+                    "Call to %s(%r) didn't fail" % (method.__name__, notTrue,))
 
+
+    def _assertTrueTrue(self, method):
+        """
+        Perform the positive case test for C{assertTrue} and C{failUnless}.
+
+        @param method: The test method to test.
+        """
+        for true in [1, True, 'cat', [1,2], (3,4)]:
+            result = method(true, "failed on %r" % (true,))
+            if result != true:
+                self.fail("Did not return argument %r" % (true,))
+
+
+    def test_assertTrueFalse(self):
+        """
+        L{SynchronousTestCase.assertTrue} raises
+        L{SynchronousTestCase.failureException} if its argument is not
+        considered true.
+        """
+        self._assertTrueFalse(self.assertTrue)
+
+
+    def test_failUnlessFalse(self):
+        """
+        L{SynchronousTestCase.failUnless} raises
+        L{SynchronousTestCase.failureException} if its argument is not
+        considered true.
+        """
+        self._assertTrueFalse(self.failUnless)
+
+
+    def test_assertTrueTrue(self):
+        """
+        L{SynchronousTestCase.assertTrue} returns its argument if its argument
+        is considered true.
+        """
+        self._assertTrueTrue(self.assertTrue)
+
+
+    def test_failUnlessTrue(self):
+        """
+        L{SynchronousTestCase.failUnless} returns its argument if its argument
+        is considered true.
+        """
+        self._assertTrueTrue(self.failUnless)
+
+
+
+class TestSynchronousAssertions(unittest.SynchronousTestCase):
+    """
+    Tests for L{SynchronousTestCase}'s assertion methods.  That is, failUnless*,
+    failIf*, assert* (not covered by other more specific test classes).
+
+    Note: As of 11.2, assertEqual is preferred over the failUnlessEqual(s)
+    variants.  Tests have been modified to reflect this preference.
+
+    This is pretty paranoid.  Still, a certain paranoia is healthy if you are
+    testing a unit testing framework.
+    """
     def _testEqualPair(self, first, second):
-        x = self.failUnlessEqual(first, second)
+        x = self.assertEqual(first, second)
         if x != first:
-            self.fail("failUnlessEqual should return first parameter")
+            self.fail("assertEqual should return first parameter")
+
 
     def _testUnequalPair(self, first, second):
         try:
-            self.failUnlessEqual(first, second)
+            self.assertEqual(first, second)
         except self.failureException, e:
             expected = 'not equal:\na = %s\nb = %s\n' % (
                 pformat(first), pformat(second))
             if str(e) != expected:
                 self.fail("Expected: %r; Got: %s" % (expected, str(e)))
         else:
-            self.fail("Call to failUnlessEqual(%r, %r) didn't fail"
+            self.fail("Call to assertEqual(%r, %r) didn't fail"
                       % (first, second))
 
-    def test_failUnlessEqual_basic(self):
+
+    def test_assertEqual_basic(self):
         self._testEqualPair('cat', 'cat')
         self._testUnequalPair('cat', 'dog')
         self._testEqualPair([1], [1])
         self._testUnequalPair([1], 'orange')
 
-    def test_failUnlessEqual_custom(self):
+
+    def test_assertEqual_custom(self):
         x = MockEquality('first')
         y = MockEquality('second')
         z = MockEquality('fecund')
@@ -112,7 +288,8 @@ class TestAssertions(unittest.TestCase):
         self._testUnequalPair(x, y)
         self._testUnequalPair(y, z)
 
-    def test_failUnlessEqualMessage(self):
+
+    def test_assertEqualMessage(self):
         """
         When a message is passed to L{assertEqual}, it is included in the
         error message.
@@ -125,7 +302,7 @@ class TestAssertions(unittest.TestCase):
             "message\nnot equal:\na = 'foo'\nb = 'bar'\n")
 
 
-    def test_failUnlessEqualNoneMessage(self):
+    def test_assertEqualNoneMessage(self):
         """
         If a message is specified as C{None}, it is not included in the error
         message of L{assertEqual}.
@@ -135,11 +312,11 @@ class TestAssertions(unittest.TestCase):
         self.assertEqual(str(exception), "not equal:\na = 'foo'\nb = 'bar'\n")
 
 
-    def test_failUnlessEqual_incomparable(self):
+    def test_assertEqual_incomparable(self):
         apple = MockEquality('apple')
         orange = ['orange']
         try:
-            self.failUnlessEqual(apple, orange)
+            self.assertEqual(apple, orange)
         except self.failureException:
             self.fail("Fail raised when ValueError ought to have been raised.")
         except ValueError:
@@ -149,6 +326,7 @@ class TestAssertions(unittest.TestCase):
             self.fail("Comparing %r and %r should have raised an exception"
                       % (apple, orange))
 
+
     def _raiseError(self, error):
         raise error
 
@@ -174,7 +352,7 @@ class TestAssertions(unittest.TestCase):
         try:
             self.failUnlessRaises(ValueError, lambda : None)
         except self.failureException, e:
-            self.failUnlessEqual(str(e),
+            self.assertEqual(str(e),
                                  'ValueError not raised (None returned)')
         else:
             self.fail("Exception not raised. Should have failed")
@@ -197,7 +375,7 @@ class TestAssertions(unittest.TestCase):
     def test_failIfEqual_basic(self):
         x, y, z = [1], [2], [1]
         ret = self.failIfEqual(x, y)
-        self.failUnlessEqual(ret, x,
+        self.assertEqual(ret, x,
                              "failIfEqual should return first parameter")
         self.failUnlessRaises(self.failureException,
                               self.failIfEqual, x, x)
@@ -209,17 +387,39 @@ class TestAssertions(unittest.TestCase):
         y = MockEquality('second')
         z = MockEquality('fecund')
         ret = self.failIfEqual(x, y)
-        self.failUnlessEqual(ret, x,
+        self.assertEqual(ret, x,
                              "failIfEqual should return first parameter")
         self.failUnlessRaises(self.failureException,
                               self.failIfEqual, x, x)
         # test when __ne__ is not defined
         self.failIfEqual(x, z, "__ne__ not defined, so not equal")
 
+
+    def test_failIfIdenticalPositive(self):
+        """
+        C{failIfIdentical} returns its first argument if its first and second
+        arguments are not the same object.
+        """
+        x = object()
+        y = object()
+        result = self.failIfIdentical(x, y)
+        self.assertEqual(x, result)
+
+
+    def test_failIfIdenticalNegative(self):
+        """
+        C{failIfIdentical} raises C{failureException} if its first and second
+        arguments are the same object.
+        """
+        x = object()
+        self.failUnlessRaises(self.failureException,
+                              self.failIfIdentical, x, x)
+
+
     def test_failUnlessIdentical(self):
         x, y, z = [1], [1], [2]
         ret = self.failUnlessIdentical(x, x)
-        self.failUnlessEqual(ret, x,
+        self.assertEqual(ret, x,
                              'failUnlessIdentical should return first '
                              'parameter')
         self.failUnlessRaises(self.failureException,
@@ -231,7 +431,7 @@ class TestAssertions(unittest.TestCase):
         x, y, z = 1.0, 1.1, 1.2
         self.failUnlessApproximates(x, x, 0.2)
         ret = self.failUnlessApproximates(x, y, 0.2)
-        self.failUnlessEqual(ret, x, "failUnlessApproximates should return "
+        self.assertEqual(ret, x, "failUnlessApproximates should return "
                              "first parameter")
         self.failUnlessRaises(self.failureException,
                               self.failUnlessApproximates, x, z, 0.1)
@@ -245,7 +445,7 @@ class TestAssertions(unittest.TestCase):
         z = 8.000002
         self.failUnlessAlmostEqual(x, x, precision)
         ret = self.failUnlessAlmostEqual(x, z, precision)
-        self.failUnlessEqual(ret, x, "failUnlessAlmostEqual should return "
+        self.assertEqual(ret, x, "failUnlessAlmostEqual should return "
                              "first parameter (%r, %r)" % (ret, x))
         self.failUnlessRaises(self.failureException,
                               self.failUnlessAlmostEqual, x, y, precision)
@@ -256,7 +456,7 @@ class TestAssertions(unittest.TestCase):
         y = 8.00001
         z = 8.000002
         ret = self.failIfAlmostEqual(x, y, precision)
-        self.failUnlessEqual(ret, x, "failIfAlmostEqual should return "
+        self.assertEqual(ret, x, "failIfAlmostEqual should return "
                              "first parameter (%r, %r)" % (ret, x))
         self.failUnlessRaises(self.failureException,
                               self.failIfAlmostEqual, x, x, precision)
@@ -269,7 +469,7 @@ class TestAssertions(unittest.TestCase):
         z = "the cat sat"
         self.failUnlessSubstring(x, x)
         ret = self.failUnlessSubstring(x, z)
-        self.failUnlessEqual(ret, x, 'should return first parameter')
+        self.assertEqual(ret, x, 'should return first parameter')
         self.failUnlessRaises(self.failureException,
                               self.failUnlessSubstring, x, y)
         self.failUnlessRaises(self.failureException,
@@ -281,16 +481,129 @@ class TestAssertions(unittest.TestCase):
         z = "the cat sat"
         self.failIfSubstring(z, x)
         ret = self.failIfSubstring(x, y)
-        self.failUnlessEqual(ret, x, 'should return first parameter')
+        self.assertEqual(ret, x, 'should return first parameter')
         self.failUnlessRaises(self.failureException,
                               self.failIfSubstring, x, x)
         self.failUnlessRaises(self.failureException,
                               self.failIfSubstring, x, z)
 
+
+    def test_assertIsInstance(self):
+        """
+        Test a true condition of assertIsInstance.
+        """
+        A = type('A', (object,), {})
+        a = A()
+        self.assertIsInstance(a, A)
+
+
+    def test_assertIsInstanceMultipleClasses(self):
+        """
+        Test a true condition of assertIsInstance with multiple classes.
+        """
+        A = type('A', (object,), {})
+        B = type('B', (object,), {})
+        a = A()
+        self.assertIsInstance(a, (A, B))
+
+
+    def test_assertIsInstanceError(self):
+        """
+        Test an error with assertIsInstance.
+        """
+        A = type('A', (object,), {})
+        B = type('B', (object,), {})
+        a = A()
+        self.assertRaises(self.failureException, self.assertIsInstance, a, B)
+
+
+    def test_assertIsInstanceErrorMultipleClasses(self):
+        """
+        Test an error with assertIsInstance and multiple classes.
+        """
+        A = type('A', (object,), {})
+        B = type('B', (object,), {})
+        C = type('C', (object,), {})
+        a = A()
+        self.assertRaises(self.failureException, self.assertIsInstance, a, (B, C))
+
+
+    def test_assertIsInstanceCustomMessage(self):
+        """
+        If L{TestCase.assertIsInstance} is passed a custom message as its 3rd
+        argument, the message is included in the failure exception raised when
+        the assertion fails.
+        """
+        exc = self.assertRaises(
+            self.failureException,
+            self.assertIsInstance, 3, str, "Silly assertion")
+        self.assertIn("Silly assertion", str(exc))
+
+
+    def test_assertNotIsInstance(self):
+        """
+        Test a true condition of assertNotIsInstance.
+        """
+        A = type('A', (object,), {})
+        B = type('B', (object,), {})
+        a = A()
+        self.assertNotIsInstance(a, B)
+
+
+    def test_assertNotIsInstanceMultipleClasses(self):
+        """
+        Test a true condition of assertNotIsInstance and multiple classes.
+        """
+        A = type('A', (object,), {})
+        B = type('B', (object,), {})
+        C = type('C', (object,), {})
+        a = A()
+        self.assertNotIsInstance(a, (B, C))
+
+
+    def test_assertNotIsInstanceError(self):
+        """
+        Test an error with assertNotIsInstance.
+        """
+        A = type('A', (object,), {})
+        a = A()
+        error = self.assertRaises(self.failureException,
+                                  self.assertNotIsInstance, a, A)
+        self.assertEqual(str(error), "%r is an instance of %s" % (a, A))
+
+
+    def test_assertNotIsInstanceErrorMultipleClasses(self):
+        """
+        Test an error with assertNotIsInstance and multiple classes.
+        """
+        A = type('A', (object,), {})
+        B = type('B', (object,), {})
+        a = A()
+        self.assertRaises(self.failureException, self.assertNotIsInstance, a, (A, B))
+
+
+    def test_assertDictEqual(self):
+        """
+        L{twisted.trial.unittest.TestCase} supports the C{assertDictEqual}
+        method inherited from the standard library in Python 2.7.
+        """
+        self.assertDictEqual({'a': 1}, {'a': 1})
+    if getattr(unittest.SynchronousTestCase, 'assertDictEqual', None) is None:
+        test_assertDictEqual.skip = (
+            "assertDictEqual is not available on this version of Python")
+
+
+
+class TestAsynchronousAssertions(unittest.TestCase):
+    """
+    Tests for L{TestCase}'s asynchronous extensions to L{SynchronousTestCase}.
+    That is, assertFailure.
+    """
     def test_assertFailure(self):
         d = defer.maybeDeferred(lambda: 1/0)
         return self.assertFailure(d, ZeroDivisionError)
 
+
     def test_assertFailure_wrongException(self):
         d = defer.maybeDeferred(lambda: 1/0)
         self.assertFailure(d, OverflowError)
@@ -298,6 +611,7 @@ class TestAssertions(unittest.TestCase):
                        lambda x: x.trap(self.failureException))
         return d
 
+
     def test_assertFailure_noException(self):
         d = defer.succeed(None)
         self.assertFailure(d, ZeroDivisionError)
@@ -305,6 +619,7 @@ class TestAssertions(unittest.TestCase):
                        lambda x: x.trap(self.failureException))
         return d
 
+
     def test_assertFailure_moreInfo(self):
         """
         In the case of assertFailure failing, check that we get lots of
@@ -319,12 +634,14 @@ class TestAssertions(unittest.TestCase):
         d.addErrback(self._checkInfo, f)
         return d
 
+
     def _checkInfo(self, assertionFailure, f):
         assert assertionFailure.check(self.failureException)
         output = assertionFailure.getErrorMessage()
         self.assertIn(f.getErrorMessage(), output)
         self.assertIn(f.getBriefTraceback(), output)
 
+
     def test_assertFailure_masked(self):
         """
         A single wrong assertFailure should fail the whole test.
@@ -346,6 +663,7 @@ class TestAssertions(unittest.TestCase):
         self.assertEqual(1, len(result.failures))
 
 
+class WarningAssertionTests(unittest.SynchronousTestCase):
     def test_assertWarns(self):
         """
         Test basic assertWarns report.
@@ -355,7 +673,7 @@ class TestAssertions(unittest.TestCase):
             return a
         r = self.assertWarns(DeprecationWarning, "Woo deprecated", __file__,
             deprecated, 123)
-        self.assertEquals(r, 123)
+        self.assertEqual(r, 123)
 
 
     def test_assertWarnsRegistryClean(self):
@@ -368,11 +686,11 @@ class TestAssertions(unittest.TestCase):
             return a
         r1 = self.assertWarns(DeprecationWarning, "Woo deprecated", __file__,
             deprecated, 123)
-        self.assertEquals(r1, 123)
+        self.assertEqual(r1, 123)
         # The warning should be raised again
         r2 = self.assertWarns(DeprecationWarning, "Woo deprecated", __file__,
             deprecated, 321)
-        self.assertEquals(r2, 321)
+        self.assertEqual(r2, 321)
 
 
     def test_assertWarnsError(self):
@@ -461,10 +779,10 @@ class TestAssertions(unittest.TestCase):
         w = Warn()
         r = self.assertWarns(DeprecationWarning, "Bar deprecated", __file__,
             w.deprecated, 321)
-        self.assertEquals(r, 321)
+        self.assertEqual(r, 321)
         r = self.assertWarns(DeprecationWarning, "Bar deprecated", __file__,
             w.deprecated, 321)
-        self.assertEquals(r, 321)
+        self.assertEqual(r, 321)
 
 
     def test_assertWarnsOnCall(self):
@@ -478,10 +796,10 @@ class TestAssertions(unittest.TestCase):
         w = Warn()
         r = self.assertWarns(DeprecationWarning, "Egg deprecated", __file__,
             w, 321)
-        self.assertEquals(r, 321)
+        self.assertEqual(r, 321)
         r = self.assertWarns(DeprecationWarning, "Egg deprecated", __file__,
             w, 321)
-        self.assertEquals(r, 321)
+        self.assertEqual(r, 321)
 
 
     def test_assertWarnsFilter(self):
@@ -493,7 +811,7 @@ class TestAssertions(unittest.TestCase):
             return a
         r = self.assertWarns(PendingDeprecationWarning, "Woo deprecated",
             __file__, deprecated, 123)
-        self.assertEquals(r, 123)
+        self.assertEqual(r, 123)
 
 
     def test_assertWarnsMultipleWarnings(self):
@@ -521,7 +839,7 @@ class TestAssertions(unittest.TestCase):
         e = self.assertRaises(self.failureException,
                 self.assertWarns, DeprecationWarning, "Woo deprecated",
                 __file__, deprecated, 123)
-        self.assertEquals(str(e), "Can't handle different warnings")
+        self.assertEqual(str(e), "Can't handle different warnings")
 
 
     def test_assertWarnsAfterUnassertedWarning(self):
@@ -540,83 +858,8 @@ class TestAssertions(unittest.TestCase):
         self.assertEqual(warning['message'], "foo")
 
 
-    def test_assertIsInstance(self):
-        """
-        Test a true condition of assertIsInstance.
-        """
-        A = type('A', (object,), {})
-        a = A()
-        self.assertIsInstance(a, A)
 
-    def test_assertIsInstanceMultipleClasses(self):
-        """
-        Test a true condition of assertIsInstance with multiple classes.
-        """
-        A = type('A', (object,), {})
-        B = type('B', (object,), {})
-        a = A()
-        self.assertIsInstance(a, (A, B))
-
-    def test_assertIsInstanceError(self):
-        """
-        Test an error with assertIsInstance.
-        """
-        A = type('A', (object,), {})
-        B = type('B', (object,), {})
-        a = A()
-        self.assertRaises(self.failureException, self.assertIsInstance, a, B)
-
-    def test_assertIsInstanceErrorMultipleClasses(self):
-        """
-        Test an error with assertIsInstance and multiple classes.
-        """
-        A = type('A', (object,), {})
-        B = type('B', (object,), {})
-        C = type('C', (object,), {})
-        a = A()
-        self.assertRaises(self.failureException, self.assertIsInstance, a, (B, C))
-
-    def test_assertNotIsInstance(self):
-        """
-        Test a true condition of assertNotIsInstance.
-        """
-        A = type('A', (object,), {})
-        B = type('B', (object,), {})
-        a = A()
-        self.assertNotIsInstance(a, B)
-
-    def test_assertNotIsInstanceMultipleClasses(self):
-        """
-        Test a true condition of assertNotIsInstance and multiple classes.
-        """
-        A = type('A', (object,), {})
-        B = type('B', (object,), {})
-        C = type('C', (object,), {})
-        a = A()
-        self.assertNotIsInstance(a, (B, C))
-
-    def test_assertNotIsInstanceError(self):
-        """
-        Test an error with assertNotIsInstance.
-        """
-        A = type('A', (object,), {})
-        a = A()
-        error = self.assertRaises(self.failureException,
-                                  self.assertNotIsInstance, a, A)
-        self.assertEquals(str(error), "%r is an instance of %s" % (a, A))
-
-    def test_assertNotIsInstanceErrorMultipleClasses(self):
-        """
-        Test an error with assertNotIsInstance and multiple classes.
-        """
-        A = type('A', (object,), {})
-        B = type('B', (object,), {})
-        a = A()
-        self.assertRaises(self.failureException, self.assertNotIsInstance, a, (A, B))
-
-
-
-class TestAssertionNames(unittest.TestCase):
+class TestAssertionNames(unittest.SynchronousTestCase):
     """
     Tests for consistency of naming within TestCase assertion methods
     """
@@ -640,14 +883,14 @@ class TestAssertionNames(unittest.TestCase):
         """
         asserts = set(self._getAsserts())
         failUnlesses = set(reflect.prefixedMethods(self, 'failUnless'))
-        self.assertEquals(
+        self.assertEqual(
             failUnlesses, asserts.intersection(failUnlesses))
 
 
     def test_failIf_matches_assertNot(self):
-        asserts = reflect.prefixedMethods(unittest.TestCase, 'assertNot')
-        failIfs = reflect.prefixedMethods(unittest.TestCase, 'failIf')
-        self.failUnlessEqual(sorted(asserts, key=self._name),
+        asserts = reflect.prefixedMethods(unittest.SynchronousTestCase, 'assertNot')
+        failIfs = reflect.prefixedMethods(unittest.SynchronousTestCase, 'failIf')
+        self.assertEqual(sorted(asserts, key=self._name),
                              sorted(failIfs, key=self._name))
 
     def test_equalSpelling(self):
@@ -657,16 +900,16 @@ class TestAssertionNames(unittest.TestCase):
             if name.endswith('Equal'):
                 self.failUnless(hasattr(self, name+'s'),
                                 "%s but no %ss" % (name, name))
-                self.failUnlessEqual(value, getattr(self, name+'s'))
+                self.assertEqual(value, getattr(self, name+'s'))
             if name.endswith('Equals'):
                 self.failUnless(hasattr(self, name[:-1]),
                                 "%s but no %s" % (name, name[:-1]))
-                self.failUnlessEqual(value, getattr(self, name[:-1]))
+                self.assertEqual(value, getattr(self, name[:-1]))
 
 
-class TestCallDeprecated(unittest.TestCase):
+class TestCallDeprecated(unittest.SynchronousTestCase):
     """
-    Test use of the L{TestCase.callDeprecated} method with version objects.
+    Test use of the L{SynchronousTestCase.callDeprecated} method with version objects.
     """
 
     version = Version('Twisted', 8, 0, 0)
